<?php
/***********************************************************
 *			HelperDB.php
 *		    --------------------
 * Created		: Wed, Feb 1, 2006
 * Copyright		: (C) LookStrike Team
 * WebSite		: http://www.lookstrike.com
 *
 * $Id: HelperDB.php,v 1.7 2007/02/15 22:21:48 jsgoupil Exp $
 *
 ***********************************************************/
class HelperDB {
	/**
	 * Fills the $instance with the $_POST
	 * $start is the "code" before the member name
	 *
	 * @param mixed $instance
	 * @param string $start
	 * @param boolean $reset
	 */
	public static function FillInstance($instance, $start, $reset = false) {
		// Reset checkbox
		if($reset === true) {
			$checkbox = $instance->getCheckbox();
			foreach ($checkbox as $key => $val) {
				$instance->$val = 0;
			}
		}

		// Get _POST data
		foreach ($_POST as $key => $val) {
			$c = strlen($start);
			if (substr($key, 0, $c) === $start) {
				$code = substr($key, $c);
				if (isset($instance->$code)) {
					$instance->$code = $val;
				}
			}
		}
	}

	/**
	 * Returns the instance of $clazzname thanks to the $primaryValue.
	 * If $fill is not empty, the FillInstance will be called.
	 *
	 * @param string $clazzname
	 * @param mixed $primaryKey
	 * @param string $fill
	 * @param boolean $reset
	 * @return db_data
	 */
	public static function GetInstance($clazzname, $primaryValue, $fill = '', $reset = false) {
		$clazz = new $clazzname;
		$clazz->fill(array($clazz->getPrimary() => $primaryValue));

		if (!empty($fill)) {
			self::FillInstance($clazz, $fill, $reset);
		}

		return $clazz;
	}

	/**
	 * Returns the login of a player
	 *
	 * @param int $player_id
	 * @return string
	 */
	public static function GetPlayerLogin($player_id) {
		return self::GetInstance('db_players', $player_id)->login;
	}

	/**
	 * Returns the nickname of a player
	 *
	 * @param int $player_id
	 * @return string
	 */
	public static function GetPlayerNickname($player_id) {
		return self::GetInstance('db_players', $player_id)->nickname;
	}

	/**
	 * Returns the id of a player
	 *
	 * @param string $login
	 * @return int
	 */
	public static function GetPlayerId($login) {
		$player = new db_players();
		$player->fill(array('login'=>$login));

		return intval($player->player_id);
	}

	/**
	 * Returns the player's teams.
	 * $single_user
	 *  * -1 : All teams
	 *  *  0 : No single_user team
	 *  *  1 : Only single_user team
	 *
	 *
	 * @param int $player_id
	 * @param int $single_user
	 * return db_teams[]
	 */
	public static function GetPlayerTeams($player_id, $single_user = -1) {
		$temp = new db_team_players();
		$team_players = db_data::searchForClass('db_team_players', array('player_id'=>$player_id), '', '', $temp->getPrimary());
		$c = count($team_players);

		$teams = array();
		$counter = 0;
		for($i = 0; $i < $c; $i++) {
			$team = new db_teams();
			$team->fill(array('team_id'=>$team_players[$i]->team_id));
			if($single_user === db_teams::TEAM_REAL && intval($team->single_user) !== db_teams::TEAM_REAL) {
				continue;
			} elseif($single_user === db_teams::TEAM_SINGLE && intval($team->single_user) !== db_teams::TEAM_SINGLE) {
				continue;
			}
			$teams[$counter++] = $team;
		}

		return $teams;
	}


	/**
	 * Returns the team's players ordered by nickname
	 *
	 * @param int $team_id
	 * return db_players[]
	 */
	public static function GetTeamPlayers($team_id) {
		$temp = new db_team_players();
		$player_teams = db_data::searchForClass('db_team_players', array('team_id'=>$team_id), '', '', $temp->getPrimary());
		$c = count($player_teams);

		$players = array();
		$counter = 0;
		for($i = 0; $i < $c; $i++) {
			$player = new db_players();
			$player->fill(array('player_id'=>$player_teams[$i]->player_id));
			$players[$player->nickname] = $player;
		}

		ksort($players);
		$player = array();
		$counter = 0;
		foreach($players as $key => $val) {
			$player[$counter++] = $val;
		}

		return $player;
	}

	/**
	 * Returns the player's tournament participations.
	 *
	 * @param int $player_id
	 * return db_tournaments[]
	 */
	public static function GetPlayerTournament($player_id) {
		$teams = self::GetPlayerTeams($player_id, db_teams::TEAM_SINGLE);

		return self::GetTeamTournament($teams[0]->team_id);
	}

	/**
	 * Returns the team's tournament participations.
	 *
	 * @param int $team_id
	 * return db_tournaments[]
	 */
	public static function GetTeamTournament($team_id) {
		$temp = new db_tournament_teams();
		$tournaments = array();
		$team_tournaments = db_data::searchForClass('db_tournament_teams', array('team_id'=>$team_id), '', '', $temp->getPrimary());
		$c = count($team_tournaments);
		for($i = 0; $i < $c; $i++) {
			$tournaments[$i] = new db_tournaments();
			$tournaments[$i]->fill(array('tournament_id'=>$team_tournaments[$i]->tournament_id));
		}
		return $tournaments;
	}

	/**
	 * Returns the teams in a tournament.
	 *
	 * @param int $tournament_id
	 * return db_teams[]
	 */
	public static function GetTournamentTeam($tournament_id) {
		$temp = new db_tournament_teams();
		$teams = array();
		$team_tournaments = db_data::searchForClass('db_tournament_teams', array('tournament_id'=>$tournament_id), '', '', $temp->getPrimary());
		$c = count($team_tournaments);
		for($i = 0; $i < $c; $i++) {
			$teams[$i] = new db_teams();
			$teams[$i]->fill(array('team_id'=>$team_tournaments[$i]->team_id));
		}
		return $teams;
	}

	/**
	 * Returns the tournament name
	 * $tournament_id can be either the id or the instance.
	 *
	 *
	 * @param mixed $tournament_id
	 */
	public static function GetTournamentName($tournament_id) {
		if($tournament_id instanceof db_tournaments) {
			$tournament = $tournament_id;
		} else {
			$tournament = new db_tournaments();
			$tournament->fill(array('tournament_id' => $tournament_id));
		}

		$game = new db_games();
		$game->fill(array('game_id'=>$tournament->game_id));

		return $game->name . ' #' . gamenumber($tournament->tournament_id, $game->game_id);
	}

	/**
	 * Returns team's name if the $team_id is a team or returns the
	 * player's name if the $team_id is a player.
	 * $team_id can be either the id or the instance.
	 *
	 * @param int $team_id
	 * @return string
	 */
	public static function GetTeamName($team_id) {
		if($team_id instanceof db_teams) {
			$team = $team_id;
		} else {
			$team = new db_teams();
			$team->fill(array('team_id' => $team_id));
		}

		if(intval($team->single_user) === 1) {	// player
			$team_players = new db_team_players();
			$team_players->fill(array('team_id' => $team->team_id ));
			return self::GetPlayerNickname( $team_players->player_id );
		} else {				// team
			return $team->name;
		}

		return '';
	}
}
?>