<?php
/***********************************************************
 *			db_games.php
 *		    --------------------
 * Created		: Wed, Feb 1, 2006
 * Copyright		: (C) LookStrike Team
 * WebSite		: http://www.lookstrike.com
 *
 * $Id: db_games.php,v 1.8 2007/06/18 01:31:01 jsgoupil Exp $
 *
 ***********************************************************/
include_once($sys_conf['path']['real'] . '/modules/class/db/db_data.php');
include_once($sys_conf['path']['real'] . '/modules/class/db/CheckDB.php');

class db_games extends db_data {
	const ERROR_GAME_NOTFOUND	= 1;
	const ERROR_PLATFORM_NOTFOUND	= 2;
	const ERROR_GAME_INUSE		= 3;

	public $game_id;		// int(11)
	public $name;			// varchar(50)
	public $description;		// text
	public $platform_id;		// int(11) -> platform.platform_id

	protected $_ERROR_LANGUAGE = array(
		self::ERROR_GAME_NOTFOUND	=> 'GAME_NOTFOUND',
		self::ERROR_PLATFORM_NOTFOUND	=> 'PLATFORM_NOTFOUND',
		self::ERROR_GAME_INUSE		=> 'GAME_INUSE'
	);

	protected $_primary = 'game_id';
	protected $_unique = array(
		'name'
	);
	protected $_required_admin = array(
		'name',
		'description',
		'platform_id'
	);
	protected $_required = array();
	protected $_filter = array();

	public function __construct() {
		$this->game_id		= 0;
		$this->name		= '';
		$this->description	= '';
		$this->platform_id	= 0;
	}

	public function insert() {
		global $db;

		$error = $this->checkData(db_data::OPERATION_INSERT);
		if ($error !== db_data::ERROR_NONE) {
			return $error;
		}

		if(CheckDB::isPlatform($this->platform_id) === false) {
			return self::ERROR_PLATFORM_NOTFOUND;
		}

		$data_insert = db_data::generateInsert('name', 'description', 'platform_id');
		$table = $this->getTableName($this);
		$db->sql_query('INSERT INTO ' . $table . ' ' . $data_insert);
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}
		$this->{$this->_primary} = $db->sql_insertid();

		return db_data::ERROR_NONE;
	}

	public function modify() {
		global $db;

		if ($this->exists() === false) {
			return self::ERROR_GAME_NOTFOUND;
		}

		$error = $this->checkData(db_data::OPERATION_MODIFY);
		if ($error !== self::ERROR_NONE) {
			return $error;
		}

		if(CheckDB::isPlatform($this->platform_id) === false) {
			return self::ERROR_PLATFORM_NOTFOUND;
		}

		$data_modify = db_data::generateModify('name', 'description', 'platform_id');
		$table = $this->getTableName($this);
		$db->sql_query('UPDATE ' . $table . ' SET ' . $data_modify . 'WHERE ' . $this->_primary . '=\'' . $this->{$this->_primary} . '\'');
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}

		return db_data::ERROR_NONE;
	}

	public function delete() {
		global $sys_conf, $db;

		if ($this->exists() === false) {
			return self::ERROR_GAME_NOTFOUND;
		}

		// Rule to follow before deleting a game :
		// - Can't delete if the game is in a tournament
		$req_game = $db->sql_query('SELECT tournament_id FROM ' . $sys_conf['db']['table_prefix'] . 'tournaments WHERE game_id=\'' . $this->game_id . '\'');
		$line_game = $db->sql_numrows($req_game);
		if ($line_game !== 0) {
			return self::ERROR_GAME_INUSE;
		}

		$table = $this->getTableName($this);
		$db->sql_query('DELETE FROM ' . $table . ' WHERE ' . $this->_primary . '=\'' . $this->{$this->_primary} . '\'');
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}

		return db_data::ERROR_NONE;
	}
}
?>