<?php
/***********************************************************
 *			db_match_teams.php
 *		    --------------------------
 * Created		: Wed, Feb 1, 2006
 * Copyright		: (C) LookStrike Team
 * WebSite		: http://www.lookstrike.com
 *
 * $Id: db_match_teams.php,v 1.2 2007/06/18 01:31:01 jsgoupil Exp $
 *
 ***********************************************************/
include_once($sys_conf['path']['real'] . '/modules/class/db/db_data.php');
include_once($sys_conf['path']['real'] . '/modules/class/db/CheckDB.php');

class db_match_teams extends db_data {
	const ERROR_MATCHPLAYER_NOTFOUND	= 1;
	const ERROR_MATCH_NOTFOUND		= 2;
	const ERROR_TEAM_NOTFOUND		= 3;

	public $match_id;		// int(11) -> matches.match_id
	public $team_id;		// int(11) -> teams.team_id
	public $score;			// int(11)

	protected $_ERROR_LANGUAGE = array(
		self::ERROR_MATCHPLAYER_NOTFOUND	=> 'MATCHPLAYER_NOTFOUND',
		self::ERROR_MATCH_NOTFOUND		=> 'MATCH_NOTFOUND',
		self::ERROR_TEAM_NOTFOUND		=> 'TEAM_NOTFOUND'
	);

	protected $_primary = array('match_id', 'team_id');
	protected $_unique = array(
		array('match_id', 'team_id')
	);
	protected $_required_admin = array();
	protected $_required = array();
	protected $_filter = array();

	public function __construct() {
		$this->match_id		= 0;
		$this->team_id		= 0;
		$this->score		= 0;
	}

	public function insert() {
		global $db;

		$error = $this->checkData(db_data::OPERATION_INSERT);
		if ($error !== db_data::ERROR_NONE) {
			return $error;
		}

		if(CheckDB::isMatch($this->match_id) === false) {
			return self::ERROR_MATCH_NOTFOUND;
		}

		if(CheckDB::isTeam($this->team_id) === false) {
			return self::ERROR_TEAM_NOTFOUND;
		}

		$table = $this->getTableName($this);
		$data_insert = db_data::generateInsert('match_id', 'team_id', 'score');
		$db->sql_query('INSERT INTO ' . $table . ' ' . $data_insert);
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}

		return db_data::ERROR_NONE;
	}

	public function modify() {
		global $db;

		if ($this->exists() === false) {
			return self::ERROR_MATCHPLAYER_NOTFOUND;
		}

		$error = $this->checkData(db_data::OPERATION_MODIFY);
		if ($error !== db_data::ERROR_NONE) {
			return $error;
		}

		if(CheckDB::isMatch($this->match_id) === false) {
			return self::ERROR_MATCH_NOTFOUND;
		}

		if(CheckDB::isTeam($this->team_id) === false) {
			return self::ERROR_TEAM_NOTFOUND;
		}

		$table = $this->getTableName($this);
		$data_modify = db_data::generateModify('score');
		$db->sql_query('UPDATE ' . $table . ' SET ' . $data_modify  . ' WHERE ' . $this->createWhere($this->_primary));
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}

		return db_data::ERROR_NONE;
	}

	public function delete() {
		global $sys_conf, $db;

		if ($this->exists() === false) {
			return self::ERROR_MATCHPLAYER_NOTFOUND;
		}

		// Rule to follow before deleting a score :
		//  - No rules

		$table = $this->getTableName($this);
		$db->sql_query('DELETE FROM ' . $table . ' WHERE ' . $this->createWhere($this->_primary));
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}

		return db_data::ERROR_NONE;
	}
}
?>