<?php
/***********************************************************
 *			db_matches.php
 *		    ----------------------
 * Created		: Wed, Feb 1, 2006
 * Copyright		: (C) LookStrike Team
 * WebSite		: http://www.lookstrike.com
 *
 * $Id: db_matches.php,v 1.9 2008/01/27 01:58:49 jsgoupil Exp $
 *
 ***********************************************************/
include_once($sys_conf['path']['real'] . '/modules/class/db/db_data.php');
include_once($sys_conf['path']['real'] . '/modules/class/db/db_tournaments.php');
include_once($sys_conf['path']['real'] . '/modules/class/db/db_match_teams.php');
include_once($sys_conf['path']['real'] . '/modules/class/db/CheckDB.php');

class db_matches extends db_data {
	const ERROR_MATCH_NOTFOUND	= 1;
	const ERROR_TOURNAMENT_NOTFOUND = 2;
	const ERROR_TREE_NOTFOUND	= 3;
	const ERROR_SERVER_NOTFOUND	= 4;
	const ERROR_ADDTOURNAMENT	= 5;

	public $match_id;		// int(11)
	public $tournament_id;		// int(11) -> tournaments.tournament_id
	public $comment;		// varchar(255)
	public $start_time;		// timestamp
	public $end_time;		// timestamp
	public $tree_id;		// int(11) -> trees.tree_id [optional]
	public $node;			// int(11) [optional]
	public $server_id;		// int(11) -> server_ip.server_id [optional]

	protected $_ERROR_LANGUAGE = array(
		self::ERROR_MATCH_NOTFOUND	=> 'MATCH_NOTFOUND',
		self::ERROR_TOURNAMENT_NOTFOUND	=> 'TOURNAMENT_NOTFOUND',
		self::ERROR_TREE_NOTFOUND	=> 'TREE_NOTFOUND',
		self::ERROR_SERVER_NOTFOUND	=> 'SERVER_NOTFOUND',
		self::ERROR_ADDTOURNAMENT	=> 'MATCH_ADDTOURNAMENT'
	);

	protected $_primary = 'match_id';
	protected $_unique = array(
		'match_id'
	);
	protected $_required_admin		= array(
		'game_id',
		'tournament_id',
		'start_time'
	);
	protected $_required = array();
	protected $_filter = array();

	public function __construct() {
		$this->match_id		= 0;
		$this->tournament_id	= 0;
		$this->comment		= '';
		$this->start_time	= date('Y-m-d H:i:s');
		$this->end_time		= '0000-00-00 00:00:00';
		$this->tree_id		= 0;
		$this->node		= 0;
		$this->server_id	= 0;
	}

	public function insert() {
		global $db;

		$error = $this->checkData(db_data::OPERATION_INSERT);
		if ($error !== db_data::ERROR_NONE) {
			return $error;
		}

		if(CheckDB::isTournament($this->tournament_id) === false) {
			return self::ERROR_TOURNAMENT_NOTFOUND;
		}

		if(intval($this->tree_id) > 0 && CheckDB::isTree($this->tree_id) === false) { // Optional
			return self::ERROR_TREE_NOTFOUND;
		}

		if(intval($this->server_id) > 0 && CheckDB::isServer($this->server_id) === false) { // Optional
			return self::ERROR_SERVER_NOTFOUND;
		}

		// Rule to follow before adding a match :
		//  - Can't add a match if the tournament is not in progress

		// Verify the tournament
		$tournament = new db_tournaments();
		$tournament->fill(array('tournament_id' => $this->tournament_id));
		if (intval($tournament->tournament_id) !== 0 && intval($tournament->status) !== db_tournaments::STATUS_PROGRESS) {
			return self::ERROR_ADDTOURNAMENT;
		}

		$this->{$this->_primary} = 0;
		$table = $this->getTableName($this);
		$data_insert = db_data::generateInsert('tournament_id', array('comment' => 'ifnull'), 'start_time', 'end_time', 'tree_id', array('node' => 'ifnull'), array('server_id' => 'ifnull'));
		$db->sql_query('INSERT INTO ' . $table . ' ' . $data_insert);
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}
		$this->{$this->_primary} = $db->sql_insertid();

		return db_data::ERROR_NONE;
	}

	public function modify() {
		global $db;

		if ($this->exists() === false) {
			return self::ERROR_MATCH_NOTFOUND;
		}

		$error = $this->checkData(db_data::OPERATION_MODIFY);
		if ($error !== db_data::ERROR_NONE) {
			return $error;
		}

		if(CheckDB::isTournament($this->tournament_id) === false) {
			return self::ERROR_TOURNAMENT_NOTFOUND;
		}

		if(intval($this->tree_id) > 0 && CheckDB::isTree($this->tree_id) === false) { // Optional
			return self::ERROR_TREE_NOTFOUND;
		}

		if(intval($this->server_id) > 0 && CheckDB::isServer($this->server_id) === false) { // Optional
			return self::ERROR_SERVER_NOTFOUND;
		}

		$table = $this->getTableName($this);
		$data_modify = db_data::generateModify('tournament_id', array('comment' => 'ifnull'), 'start_time', 'end_time', 'tree_id', array('node' => 'ifnull'), array('server_id' => 'ifnull'));
		$db->sql_query('UPDATE ' . $table . ' SET ' . $data_modify  . ' WHERE ' . $this->_primary . '=\'' . $this->{$this->_primary} . '\'');
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}

		return db_data::ERROR_NONE;
	}

	public function delete() {
		global $sys_conf, $db;

		if ($this->exists() === false) {
			return self::ERROR_MATCH_NOTFOUND;
		}

		// Rule to follow before deleting a match :
		//  - No rules

		// Delete match_teams
		$match_teams = db_data::searchFor('match_teams', array('match_id' => $this->match_id));
		$c = count($match_teams);
		for ($i = 0; $i < $c; $i++) {
			$match_player = new db_match_teams();
			$match_player->fill(array('AND' => array('match_id' => $this->match_id, 'team_id' => $match_teams[$i]['team_id'])));
			$match_player->delete();
		}

		$table = $this->getTableName($this);
		$db->sql_query('DELETE FROM ' . $table . ' WHERE ' . $this->_primary . '=\'' . $this->{$this->_primary} . '\'');
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}

		return db_data::ERROR_NONE;
	}
}
?>