<?php
/***********************************************************
 *			db_platform.php
 *		    -----------------------
 * Created		: Wed, Feb 1, 2006
 * Copyright		: (C) LookStrike Team
 * WebSite		: http://www.lookstrike.com
 *
 * $Id: db_platform.php,v 1.4 2008/01/19 13:35:47 jsgoupil Exp $
 *
 ***********************************************************/
include_once($sys_conf['path']['real'] . '/modules/class/db/db_data.php');

class db_platform extends db_data {
	const ERROR_PLATFORM_NOTFOUND	= 1;
	const ERROR_PLATFORM_INUSE	= 2;

	public $platform_id;		// int(11)
	public $name;			// varchar(255)
	public $tag;			// varchar(255)
	public $image;			// varchar(255)

	protected $_ERROR_LANGUAGE = array(
		self::ERROR_PLATFORM_NOTFOUND	=> 'PLATFORM_NOTFOUND',
		self::ERROR_PLATFORM_INUSE	=> 'PLATFORM_INUSE'
	);

	protected $_primary = 'platform_id';
	protected $_unique = array(
		'name'
	);
	protected $_required_admin = array(
		'name',
		'tag',
		'image'
	);
	protected $_required = array();
	protected $_filter = array();

	public function __construct() {
		$this->platform_id	= 0;
		$this->name		= '';
		$this->tag		= '';
		$this->image		= '';
	}

	public function insert() {
		global $db;

		$error = $this->checkData(db_data::OPERATION_INSERT);
		if ($error !== db_data::ERROR_NONE) {
			return $error;
		}

		$this->{$this->_primary} = 0;
		$table = $this->getTableName($this);
		$data_insert = db_data::generateInsert('name', 'tag', 'image');
		$db->sql_query('INSERT INTO ' . $table . ' ' . $data_insert);
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}
		$this->{$this->_primary} = $db->sql_insertid();

		return db_data::ERROR_NONE;
	}

	public function modify() {
		global $db;

		if ($this->exists() === false) {
			return self::ERROR_PLATFORM_NOTFOUND;
		}

		$error = $this->checkData(db_data::OPERATION_MODIFY);
		if ($error !== db_data::ERROR_NONE) {
			return $error;
		}

		$data_modify = db_data::generateModify('name', 'tag', 'image');
		$table = $this->getTableName($this);
		$db->sql_query('UPDATE ' . $table . ' SET ' . $data_modify . ' WHERE ' . $this->_primary . '=\'' . $this->{$this->_primary} . '\'');
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}

		return db_data::ERROR_NONE;
	}

	public function delete() {
		global $sys_conf, $db;

		if ($this->exists() === false) {
			return self::ERROR_PLATFORM_NOTFOUND;
		}

		// Rule to follow before deleting a platform :
		// - Can't delete if a game in this platform is in a tournament
		$table = $this->getTableName($this);
		$req_platform = $db->sql_query('SELECT b.tournament_id FROM ' . $table . ' a, ' . $sys_conf['db']['table_prefix'] . 'tournaments b, ' . $sys_conf['db']['table_prefix'] . 'games c WHERE a.' . $this->_primary . '=\'' . $this->{$this->_primary} . '\' AND a.platform_id=c.platform_id AND b.game_id=c.game_id');
		$line_platform = $db->sql_numrows($req_platform);
		if ($line_platform !== 0) {
			return self::ERROR_PLATFORM_INUSE;
		}

		// Delete the game's
		$games = db_data::searchForClass('db_games', array($this->_primary => $this->{$this->_primary}), '', '', 'game_id');
		foreach ($games as $del) {
			$del->delete();
		}
		$db->sql_query('DELETE FROM ' . $table . ' WHERE ' . $this->_primary . '=\'' . $this->{$this->_primary} . '\'');
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}

		return db_data::ERROR_NONE;
	}
}
?>