<?php
/***********************************************************
 *			db_players.php
 *		    ----------------------
 * Created		: Wed, Feb 1, 2006
 * Copyright		: (C) LookStrike Team
 * WebSite		: http://www.lookstrike.com
 *
 * $Id: db_players.php,v 1.13 2008/02/09 20:34:57 jsgoupil Exp $
 *
 ***********************************************************/
include_once($sys_conf['path']['real'] . '/modules/class/db/db_data.php');
include_once($sys_conf['path']['real'] . '/modules/class/db/db_tournament_teams.php');
include_once($sys_conf['path']['real'] . '/modules/class/db/db_teams.php');
include_once($sys_conf['path']['real'] . '/modules/class/db/db_team_players.php');
include_once($sys_conf['path']['real'] . '/modules/class/db/db_news.php');
include_once($sys_conf['path']['real'] . '/modules/class/db/db_admins.php');

class db_players extends db_data {
	const ERROR_USER_NOTFOUND	= 1;
	const ERROR_DELETETEAM		= 2;

	public $player_id;		// int(11)
	public $login;			// varchar(15)
	public $password;		// varchar(32)
	public $nickname;		// varchar(50)
	public $firstname;		// varchar(50)
	public $lastname;		// varchar(50)
	public $email;			// varchar(70)
	public $address1;		// varchar(255)
	public $address2;		// varchar(255)
	public $city;			// varchar(50)
	public $state;			// varchar(50)
	public $country;		// varchar(50)
	public $postal_code;		// varchar(50)
	public $phone;			// varchar(50)
	public $birthday;		// date
	public $warn_user;		// tinyint(1)
	public $allow_change;		// tinyint(1)
	public $checked;		// tinyint(1)
	public $language;		// char(2)
	public $cost;			// decimal(5,2)
	public $msn;			// varchar(255)
	public $activated;		// tinyint(1)
	public $subscribed;		// timestamp
	public $last_move;		// timestamp

	protected $_ERROR_LANGUAGE = array(
		self::ERROR_USER_NOTFOUND	=> 'USER_NOTFOUND',
		self::ERROR_DELETETEAM		=> 'USER_DELETETEAM'
	);

	protected $_primary = 'player_id';
	protected $_unique = array(
		'login',
		'nickname',
		'email'
	);
	protected $_required_admin = array(
		'login'
	);
	protected $_required = array(
		'login',
		'nickname',
		'password',
		'lastname',
		'firstname',
		'email',
		'birthday'
	);
	protected $_filter = array(
		'birthday'=>'date'
	);
	protected $_checkbox = array(
		'warn_user',
		'allow_change',
		'checked'
	);

	public function __construct() {
		global $sys_conf;

		$this->player_id	= 0;
		$this->login		= '';
		$this->password		= '';
		$this->nickname		= '';
		$this->firstname	= '';
		$this->lastname		= '';
		$this->email		= '';
		$this->address1		= '';
		$this->address2		= '';
		$this->city		= '';
		$this->state		= '';
		$this->country		= '';
		$this->postal_code	= '';
		$this->phone		= '';
		$this->birthday		= date('Y-m-d');
		$this->warn_user	= 0;
		$this->allow_change	= 0;
		$this->checked		= 0;
		$this->language		= $sys_conf['lang']['default'];
		$this->cost		= 0.0;
		$this->msn		= '';
		$this->activated	= 0;
		$this->subscribed	= date('Y-m-d H:i:s');
		$this->last_move	= date('Y-m-d H:i:s');
	}

	public function insert() {
		global $sys_conf, $db;

		if ($this->nickname === '') {
			$this->nickname = self::CopyLoginToNickname($this->login);
		}

		$error = $this->checkData(db_data::OPERATION_INSERT);
		if ($error !== db_data::ERROR_NONE) {
			return $error;
		}

		$this->{$this->_primary} = 0;
		$table = $this->getTableName($this);
		$data_insert = db_data::generateInsert('login', array('password' => 'md5'), 'nickname', 'lastname', 'firstname', 'email', 'address1', array('address2' => 'ifnull'), 'city', 'state', 'country', 'postal_code', 'phone', 'birthday', array('subscribed' => 'now'), 'warn_user', 'allow_change', 'checked', 'language', 'cost', 'msn', 'activated');
		$db->sql_query('INSERT INTO ' . $table . ' ' . $data_insert);
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}
		$this->player_id = $db->sql_insertid();

		$db->sql_query('INSERT INTO ' . $sys_conf['db']['table_prefix'] . 'teams (name, password, subscribed, single_user) VALUES (NULL, NULL, NOW(), \'1\')');
		if ($db->sql_goterror() === true) {
			$db->sql_query('DELETE FROM ' . $table . ' WHERE ' . $this->_primary . '=\'' . $this->{$this->_primary} . '\'');
			$this->player_id = 0;
			return db_data::ERROR_SQL;
		}
		$team_id = $db->sql_insertid();

		$db->sql_query('INSERT INTO ' . $sys_conf['db']['table_prefix'] . 'team_players (team_id, player_id, level, subscribed) VALUES (\'' . $team_id . '\', \'' . $this->player_id . '\', \'0\', NOW())');
		if ($db->sql_goterror() === true) {
			$db->sql_query('DELETE FROM ' . $sys_conf['db']['table_prefix'] . 'teams WHERE team_id=\'' . $team_id . '\'');
			$db->sql_query('DELETE FROM ' . $table . ' WHERE ' . $this->_primary . '=\'' . $this->{$this->_primary} . '\'');
			$this->player_id = 0;
			return db_data::ERROR_SQL;
		}

		return db_data::ERROR_NONE;
	}

	public function modify() {
		global $db;

		if ($this->nickname === '') {
			$this->nickname = self::CopyLoginToNickname($this->login);
		}

		if ($this->exists() === false) {
			return self::ERROR_USER_NOTFOUND;
		}

		$error = $this->checkData(db_data::OPERATION_MODIFY);
		if ($error !== db_data::ERROR_NONE) {
			return $error;
		}

		$table = $this->getTableName($this);
		$data_modify = db_data::generateModify('nickname', array('password' => 'md5'), 'lastname', 'firstname', 'email', 'address1', array('address2' => 'ifnull'), 'city', 'state', 'country', 'postal_code', 'phone', 'birthday', 'warn_user', 'allow_change', 'checked', array('subscribed' => 'same'), array('last_move' => 'same'), 'language', 'cost', 'msn', 'activated');
		$db->sql_query('UPDATE ' . $table . ' SET ' . $data_modify  . ' WHERE ' . $this->_primary . '=\'' . $this->{$this->_primary} . '\'');
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}

		return db_data::ERROR_NONE;
	}

	public function delete() {
		global $sys_conf, $db;

		if ($this->exists() === false) {
			return self::ERROR_USER_NOTFOUND;
		}

		// Rule to follow before deleting a player :
		//  - Can't delete if a team you are subcribed at is in a tournament or if you are participating in a ffa tournament

		// We search for participating teams
		$req_team = $db->sql_query('SELECT a.team_id, b.single_user FROM ' . $sys_conf['db']['table_prefix'] . 'team_players a, ' . $sys_conf['db']['table_prefix'] . 'teams b WHERE a.player_id=\'' . $this->player_id . '\' AND a.team_id=b.team_id'); // All teams... even single_user=1
		$line_team = $db->sql_numrows($req_team);
		$data_team = $db->sql_fetchrowset($req_team);
		$tournament_team = false;
		for ($i = 0; $i < $line_team; $i++) {
			// Are these teams in a tournament ?
			$tournamentClazz = db_data::searchFor('tournament_teams', array('team_id' => $data_team[$i]['team_id']), '', '', true);
			if ($tournamentClazz !== NULL) { // We have found something...
				$tournament_team = true;
				break;
			}
		}
		if ($tournament_team === true) {
			return self::ERROR_DELETETEAM;
		}

		// We can delete you and your teams' attachment... see you later alligator
		// Can't be undone !

		// Delete team attachment or delete team if single_user=1
		for ($i = 0; $i < $line_team; $i++) {
			if (intval($data_team[$i]['single_user']) === 1) {
				$team = new db_teams();
				$team->fill(array('team_id' => $data_team[$i]['team_id']));
			} else {
				$team = new db_team_players();
				$team->fill(array('AND' => array('team_id' => $data_team[$i]['team_id'], 'player_id' => $this->player_id)));
			}
			$team->delete();
		}

		// Delete admin rights
		$admin = new db_admins();
		$admin->fill(array('player_id' => $this->player_id));
		$admin->delete();

		// Makes news Unknown sender
		$news = db_data::searchForClass('db_news', array('player_id' => $this->player_id), '', '', 'news_id');
		$c = count($news);
		for ($i = 0; $i < $c; $i++) {
			$news[$i]->player_id = 0;
			$news[$i]->modify();
		}

		// If the user was logged, we will kick him out
		$db->sql_query('UPDATE ' . $sys_conf['db']['table_prefix'] . 'sessions SET player_id=\'0\' WHERE player_id=\'' . $this->player_id . '\'');

		$table = $this->getTableName($this);
		$db->sql_query('DELETE FROM ' . $table . ' WHERE ' . $this->_primary . '=\'' . $this->{$this->_primary} . '\'');
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}

		return db_data::ERROR_NONE;
	}

	/**
	 * Returns the login
	 * If it is already taken as nickname, a number is added at end
	 *
	 * @param string $login
	 * @return string
	 */
	private static function CopyLoginToNickname($login) {
		global $db, $sys_conf;

		$i = 0;
		while (1) {
			$nick = $login . ($i === 0 ? '' : $i);
			$exist_result = $db->sql_query('SELECT nickname FROM ' . $sys_conf['db']['table_prefix'] . 'players WHERE nickname=\'' . addslashes($nick) . '\'');
			$exist_line = $db->sql_numrows($exist_result);
			if ($exist_line === 0) {
				break;
			} else {
				$i++;
			}
		}

		return $nick;
	}
}
?>