<?php
/***********************************************************
 *			db_server_group.php
 *		    ---------------------------
 * Created		: Wed, Feb 1, 2006
 * Copyright		: (C) LookStrike Team
 * WebSite		: http://www.lookstrike.com
 *
 * $Id: db_server_group.php,v 1.7 2008/01/19 13:35:47 jsgoupil Exp $
 *
 ***********************************************************/
include_once($sys_conf['path']['real'] . '/modules/class/db/db_data.php');
include_once($sys_conf['path']['real'] . '/modules/class/db/db_tournaments.php');

class db_server_group extends db_data {
	const ERROR_SERVERGROUP_NOTFOUND	= 1;
	const ERROR_SERVERGROUP_INUSE		= 2;

	public $group_id;		// int(11)
	public $name;			// varchar(50)

	protected $_ERROR_LANGUAGE = array(
		self::ERROR_SERVERGROUP_NOTFOUND	=> 'GROUP_NOTFOUND',
		self::ERROR_SERVERGROUP_INUSE		=> 'GROUP_INUSE'
	);

	protected $_primary = 'group_id';
	protected $_unique = array(
		'name'
	);
	protected $_required_admin  = array(
		'name'
	);
	protected $_required = array();
	protected $_filter = array();

	public function __construct() {
		$this->group_id		= 0;
		$this->name		= '';
	}

	public function insert() {
		global $db;

		$error = $this->checkData(db_data::OPERATION_INSERT);
		if ($error !== db_data::ERROR_NONE) {
			return $error;
		}

		$this->{$this->_primary} = 0;
		$table = $this->getTableName($this);
		$data_insert = db_data::generateInsert('name');
		$db->sql_query('INSERT INTO ' . $table . ' ' . $data_insert);
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}
		$this->{$this->_primary} = $db->sql_insertid();

		return db_data::ERROR_NONE;
	}

	public function modify() {
		global $db;

		if ($this->exists() === false) {
			return self::ERROR_SERVERGROUP_NOTFOUND;
		}

		$error = $this->checkData(db_data::OPERATION_MODIFY);
		if ($error !== db_data::ERROR_NONE) {
			return $error;
		}

		$data_modify = db_data::generateModify('name');
		$table = $this->getTableName($this);
		$db->sql_query('UPDATE ' . $table . ' SET ' . $data_modify . ' WHERE ' . $this->_primary . '=\'' . $this->{$this->_primary} . '\'');
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}

		return db_data::ERROR_NONE;
	}

	public function delete() {
		global $sys_conf, $db;

		if ($this->exists() === false) {
			return self::ERROR_SERVERGROUP_NOTFOUND;
		}

		// Rule to follow before deleting a player :
		//  - Can't delete if the group has a server
		$req_group = $db->sql_query('SELECT server_id FROM ' . $sys_conf['db']['table_prefix'] . 'server_ip WHERE group_id=\'' . $this->group_id . '\'');
		$line_group = $db->sql_numrows($req_group);
		if ($line_group !== 0) {
			return self::ERROR_SERVERGROUP_INUSE;
		}

		// Makes tournament -> no server_group
		$tournaments = db_data::searchForClass('db_tournaments', array('server_group' => $this->group_id), '', '', 'tournament_id');
		$c = count($tournaments);
		for ($i = 0; $i < $c; $i++) {
			$tournaments[$i]->server_group = 0;
			$tournaments[$i]->modify();
		}

		$table = $this->getTableName($this);
		$db->sql_query('DELETE FROM ' . $table . ' WHERE ' . $this->_primary . '=\'' . $this->{$this->_primary} . '\'');
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}

		return db_data::ERROR_NONE;
	}
}
?>