<?php
/***********************************************************
 *			db_server_ip.php
 *		    ------------------------
 * Created		: Wed, Feb 1, 2006
 * Copyright		: (C) LookStrike Team
 * WebSite		: http://www.lookstrike.com
 *
 * $Id: db_server_ip.php,v 1.6 2008/01/19 13:35:47 jsgoupil Exp $
 *
 ***********************************************************/
include_once($sys_conf['path']['real'] . '/modules/class/db/db_data.php');

class db_server_ip extends db_data {
	const ERROR_SERVERIP_NOTFOUND		= 1;
	const ERROR_SERVERGROUP_NOTFOUND	= 2;
	const ERROR_SERVERIP_INUSE		= 3;

	public $server_id;		// int(11)
	public $group_id;		// int(11) -> server_group.group_id
	public $ip;			// varchar(8)
	public $port;			// int(5)
	public $password;		// varchar(255)

	protected $_ERROR_LANGUAGE = array(
		self::ERROR_SERVERIP_NOTFOUND		=> 'SERVER_NOTFOUND',
		self::ERROR_SERVERGROUP_NOTFOUND	=> 'GROUP_NOTFOUND',
		self::ERROR_SERVERIP_INUSE		=> 'SERVER_INUSE'
	);

	protected $_primary = 'server_id';
	protected $_unique = array();
	protected $_required_admin = array(
		'group_id',
		'ip',
		'port'
	);
	protected $_required = array();
	protected $_filter = array();

	public function __construct() {
		$this->server_id	= 0;
		$this->group_id		= 0;
		$this->ip		= '';
		$this->port		= 0;
		$this->password		= '';
	}

	public function insert() {
		global $db;

		$error = $this->checkData(db_data::OPERATION_INSERT);
		if ($error !== db_data::ERROR_NONE) {
			return $error;
		}

		if(CheckDB::isServerGroup($this->group_id) === false) {
			return self::ERROR_SERVERGROUP_NOTFOUND;
		}

		$this->{$this->_primary} = 0;
		$table = $this->getTableName($this);
		$data_insert = db_data::generateInsert('group_id', 'ip', 'port', 'password');
		$db->sql_query('INSERT INTO ' . $table . ' ' . $data_insert);
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}
		$this->{$this->_primary} = $db->sql_insertid();

		return db_data::ERROR_NONE;
	}

	public function modify() {
		global $db;

		if ($this->exists() === false) {
			return self::ERROR_SERVERIP_NOTFOUND;
		}

		$error = $this->checkData(db_data::OPERATION_MODIFY);
		if ($error != db_data::ERROR_NONE) {
			return $error;
		}

		if(CheckDB::isServerGroup($this->group_id) === false) {
			return self::ERROR_SERVERGROUP_NOTFOUND;
		}

		$data_modify = db_data::generateModify('group_id', 'ip', 'port', 'password');
		$table = $this->getTableName($this);
		$db->sql_query('UPDATE ' . $table . ' SET ' . $data_modify . ' WHERE ' . $this->_primary . '=\'' . $this->{$this->_primary} . '\'');
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}

		return db_data::ERROR_NONE;
	}

	public function delete() {
		global $sys_conf, $db;

		if ($this->exists() === false) {
			return self::ERROR_SERVERIP_NOTFOUND;
		}

		// Rule to follow before deleting a server :
		//  - Can't delete if a match has this server assigned
		$req_match = $db->sql_query('SELECT match_id FROM ' . $sys_conf['db']['table_prefix'] . 'matches WHERE server_id=\'' . $this->server_id . '\'');
		$line_match = $db->sql_numrows($req_match);
		if ($line_match !== 0) {
			return self::ERROR_SERVERIP_INUSE;
		}

		$table = $this->getTableName($this);
		$db->sql_query('DELETE FROM ' . $table . ' WHERE ' . $this->_primary . '=\'' . $this->{$this->_primary} . '\'');
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}

		return db_data::ERROR_NONE;
	}
}
?>