<?php
/***********************************************************
 *			db_team_players.php
 *		    ---------------------------
 * Created		: Wed, Feb 1, 2006
 * Copyright		: (C) LookStrike Team
 * WebSite		: http://www.lookstrike.com
 *
 * $Id: db_team_players.php,v 1.7 2008/01/19 13:35:47 jsgoupil Exp $
 *
 ***********************************************************/
include_once($sys_conf['path']['real'] . '/modules/class/db/db_data.php');
include_once($sys_conf['path']['real'] . '/modules/class/db/db_tournament_teams.php');
include_once($sys_conf['path']['real'] . '/modules/class/db/db_team_players.php');
include_once($sys_conf['path']['real'] . '/modules/class/db/CheckDB.php');

class db_team_players extends db_data {
	const ERROR_NOTFOUND		= 1;
	const ERROR_TEAM_NOTFOUND	= 2;
	const ERROR_PLAYER_NOTFOUND	= 3;
	const ERROR_DELETETOURNAMENT	= 4;
	const ERROR_INSERTTOURNAMENT	= 5;

	const LEVEL_MEMBER		= 0;
	const LEVEL_CAPTAIN		= 1;

	public $team_id;		// int(11) -> teams.team_id
	public $player_id;		// int(11) -> players.player_id
	public $level;			// tinyint(1)
	public $subscribed;		// timestamp

	protected $_ERROR_LANGUAGE = array(
		self::ERROR_NOTFOUND		=> 'TEAMPLAYER_NOTFOUND',
		self::ERROR_TEAM_NOTFOUND	=> 'TEAM_NOTFOUND',
		self::ERROR_PLAYER_NOTFOUND	=> 'PLAYER_NOTFOUND',
		self::ERROR_DELETETOURNAMENT	=> 'TEAMPLAYER_DELETETOURNAMENT',
		self::ERROR_INSERTTOURNAMENT	=> 'TEAMPLAYER_INSERTTOURNAMENT'
	);

	protected $_primary = array('team_id', 'player_id');
	protected $_unique = array(
		array('team_id', 'player_id')
	);
	protected $_required_admin = array();
	protected $_required = array();
	protected $_filter = array();

	public function __construct() {
		$this->team_id		= 0;
		$this->player_id	= 0;
		$this->level		= self::LEVEL_MEMBER;
		$this->subscribed	= date('Y-m-d H:i:s');
	}

	public function insert() {
		global $db;

		$error = $this->checkData(db_data::OPERATION_INSERT);
		if ($error !== db_data::ERROR_NONE) {
			return $error;
		}

		if(CheckDB::isTeam($this->team_id) === false) {
			return self::ERROR_TEAM_NOTFOUND;
		}

		if(CheckDB::isUser($this->player_id) === false) {
			return self::ERROR_PLAYER_NOTFOUND;
		}

		// Rule to follow before adding a player :
		//  - The team must not be in tournament

		// We search if the team is participating a tournament
		$tournamentClazz = db_data::searchFor('tournament_teams', array('team_id' => $this->team_id), '', '', true);
		if ($tournamentClazz !== NULL) {
			return self::ERROR_INSERTTOURNAMENT;
		}

		$table = $this->getTableName($this);
		$data_insert = db_data::generateInsert('team_id', 'player_id', 'level', array('subscribed' => 'now'));
		$db->sql_query('INSERT INTO ' . $table . ' ' . $data_insert);
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}

		// Remove previous leader if this one is a new leader
		if(intval($this->level) === self::LEVEL_CAPTAIN) {
			$this->changeLeader($this->player_id);
		}

		return db_data::ERROR_NONE;
	}

	public function modify() {
		global $db;

		// We check if we already have the record, we can't change the team_id and player_id
		if ($this->exists() === false) {
			return self::ERROR_NOTFOUND;
		}

		$error = $this->checkData(db_data::OPERATION_MODIFY);
		if ($error !== db_data::ERROR_NONE) {
			return $error;
		}

		$table = $this->getTableName($this);
		$data_modify = db_data::generateModify('level', array('subscribed' => 'same'));
		$db->sql_query('UPDATE ' . $table . ' SET ' . $data_modify  . ' WHERE ' . $this->createWhere($this->_primary));
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}

		// Remove previous leader if this one is a new leader
		if(intval($this->level) === self::LEVEL_CAPTAIN) {
			$this->changeLeader($this->player_id);
		}

		return db_data::ERROR_NONE;
	}

	public function delete() {
		global $sys_conf, $db;

		if ($this->exists() === false) {
			return self::ERROR_NOTFOUND;
		}

		// Rule to follow before deleting a player :
		//  - Can't delete if the team pointed is in a tournament

		// We search if the team is participating a tournament
		$tournamentClazz = db_data::searchFor('tournament_teams', array('team_id' => $this->team_id), '', '', true);
		if ($tournamentClazz !== NULL) {
			return self::ERROR_DELETETOURNAMENT;
		}

		// We can delete the player
		// Can't be undone !

		$table = $this->getTableName($this);
		$db->sql_query('DELETE FROM ' . $table . ' WHERE ' . $this->createWhere($this->_primary));
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}

		// We are clever :) When we have deleted the captain,
		// We will promote freely the older player in the team
		if (intval($this->level) === self::LEVEL_CAPTAIN) {
			$team_player = new db_team_players();
			$team_player->fill(array('team_id' => $this->team_id), array('subscribed' => 'ASC'), array(1, 0));
			// Found someone ?
			if (intval($team_player->player_id) !== 0) {
				$team_player->level = self::LEVEL_CAPTAIN;
				$team_player->modify();
			}
		}

		return db_data::ERROR_NONE;
	}

	/**
	 * Removes the previous leader from the team.
	 * The $new_leader is the id of the new leader
	 *
	 * @param int $new_leader
	 */
	private function changeLeader($new_leader) {
		global $db;

		// We remove the leader from the team
		$table = $this->getTableName($this);
		$db->sql_query('UPDATE ' . $table . ' SET `level`=\'' . self::LEVEL_MEMBER . '\', `subscribed`=`subscribed` WHERE team_id=\'' . $this->team_id . '\' AND player_id!=\'' . $new_leader . '\'');
	}
}
?>