<?php
/***********************************************************
 *			db_teams.php
 *		    --------------------
 * Created		: Wed, Feb 1, 2006
 * Copyright		: (C) LookStrike Team
 * WebSite		: http://www.lookstrike.com
 *
 * $Id: db_teams.php,v 1.6 2007/02/15 22:23:30 jsgoupil Exp $
 *
 ***********************************************************/
include_once($sys_conf['path']['real'] . '/modules/class/db/db_data.php');
include_once($sys_conf['path']['real'] . '/modules/class/db/db_tournament_teams.php');
include_once($sys_conf['path']['real'] . '/modules/class/db/db_team_players.php');

class db_teams extends db_data {
	const ERROR_NOTFOUND		= 1;
	const ERROR_DELETETOURNAMENT	= 2;

	const TEAM_REAL			= 0;
	const TEAM_SINGLE		= 1;

	public $team_id;		// int(11)
	public $name;			// varchar(50)
	public $password;		// varchar(32)
	public $single_user;		// tinyint(1)
	public $subscribed;		// timestamp

	protected $_ERROR_LANGUAGE = array(
		self::ERROR_NOTFOUND		=> 'TEAM_NOTFOUND',
		self::ERROR_DELETETOURNAMENT	=> 'TEAM_DELETETOURNAMENT'
	);

	protected $_primary = 'team_id';
	protected $_unique = array(
		'name'
	);
	protected $_required_admin = array(
		'name',
		'password'
	);
	protected $_required = array(
		'name',
		'password'
	);
	protected $_filter = array();

	public function __construct() {
		$this->team_id		= 0;
		$this->name		= '';
		$this->password		= '';
		$this->single_user	= 0;
		$this->subscribed	= date('Y-m-d H:i:s');
	}

	public function insert() {
		global $db;

		$error = $this->checkData(db_data::OPERATION_INSERT);
		if ($error !== db_data::ERROR_NONE) {
			return $error;
		}

		$this->{$this->_primary} = 0;
		$table = $this->getTableName($this);
		// The name cannot be null because it is only for team with single_user and we let the player to create them.
		$data_insert = db_data::generateInsert('name', array('password' => 'md5'), 'single_user', array('subscribed' => 'now'));
		$db->sql_query('INSERT INTO ' . $table . ' ' . $data_insert);
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}
		$this->{$this->_primary} = $db->sql_insertid();

		return db_data::ERROR_NONE;
	}

	public function modify() {
		global $db;

		if ($this->exists() === false) {
			return self::ERROR_NOTFOUND;
		}

		$error = $this->checkData(db_data::OPERATION_MODIFY);
		if ($error !== db_data::ERROR_NONE) {
			return $error;
		}

		$table = $this->getTableName($this);
		$data_modify = db_data::generateModify('name', array('password' => 'md5'), 'single_user', array('subscribed' => 'same'));
		$db->sql_query('UPDATE ' . $table . ' SET ' . $data_modify  . ' WHERE ' . $this->_primary . '=\'' . $this->{$this->_primary} . '\'');
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}

		return db_data::ERROR_NONE;
	}

	public function delete() {
		global $sys_conf, $db;

		if ($this->exists() === false) {
			return self::ERROR_NOTFOUND;
		}

		// Rule to follow before deleting a team :
		//  - Can't delete if the team is in a tournament

		// We search if team is participating a tournament
		$tournamentClazz = db_data::searchFor('tournament_teams', array('team_id' => $this->team_id), '', '', true);
		if ($tournamentClazz !== NULL) {
			return self::ERROR_DELETETOURNAMENT;
		}

		// We can delete the team
		// Can't be undone !

		// Delete player attachment
		// Ordering by level to be sure that the LAST one deleted will be the captain. So we will not do an update to give the captain level to anyone.
		$req = $db->sql_query('SELECT player_id FROM ' . $sys_conf['db']['table_prefix'] . 'team_players WHERE team_id=\'' . $this->team_id . '\' ORDER BY level ASC');
		$line = $db->sql_numrows($req);
		$data = $db->sql_fetchrowset($req);
		for ($i = 0; $i < $line; $i++) {
			$team_player = new db_team_players();
			$team_player->fill(array('AND' => array('team_id' => $this->team_id, 'player_id' => $data[$i]['player_id'])));
			$team_player->delete();
		}

		$table = $this->getTableName($this);
		$db->sql_query('DELETE FROM ' . $table . ' WHERE ' . $this->_primary . '=\'' . $this->{$this->_primary} . '\'');
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}

		return db_data::ERROR_NONE;
	}
}
?>