<?php
/***********************************************************
 *			db_tournament_teams.php
 *		    -------------------------------
 * Created		: Wed, Feb 1, 2006
 * Copyright		: (C) LookStrike Team
 * WebSite		: http://www.lookstrike.com
 *
 * $Id: db_tournament_teams.php,v 1.6 2007/06/18 07:22:55 jsgoupil Exp $
 *
 ***********************************************************/
include_once($sys_conf['path']['real'] . '/modules/class/db/db_data.php');
include_once($sys_conf['path']['real'] . '/modules/class/db/db_tournaments.php');
include_once($sys_conf['path']['real'] . '/modules/class/db/CheckDB.php');

class db_tournament_teams extends db_data {
	const ERROR_NOTFOUND		= 1;
	const ERROR_TOURNAMENT_NOTFOUND	= 2;
	const ERROR_TEAM_NOTFOUND	= 3;
	const ERROR_DELETETOURNAMENT	= 4;
	const ERROR_INSERTTOURNAMENT	= 5;

	public $tournament_id;		// int(11) -> tournaments.tournament_id
	public $team_id;		// int(11) -> teams.team_id
	public $subscribed;		// timestamp

	protected $_ERROR_LANGUAGE = array(
		self::ERROR_NOTFOUND		=> 'TOURNTEAM_NOTFOUND',
		self::ERROR_TOURNAMENT_NOTFOUND	=> 'TOURNAMENT_NOTFOUND',
		self::ERROR_TEAM_NOTFOUND	=> 'TEAM_NOTFOUND',
		self::ERROR_DELETETOURNAMENT	=> 'TOURNTEAM_DELETETOURNAMENT',
		self::ERROR_INSERTTOURNAMENT	=> 'TOURNTEAM_INSERTTOURNAMENT'
	);

	protected $_primary = array('tournament_id', 'team_id');
	protected $_unique = array(
		array('tournament_id', 'team_id')
	);
	protected $_required_admin		= array(
		'tournament_id',
		'team_id'
	);
	protected $_required = array();
	protected $_filter = array();

	public function __construct() {
		$this->tournament_id	= 0;
		$this->team_id		= 0;
		$this->subscribed	= date('Y-m-d H:i:s');
	}

	public function insert() {
		global $db;

		$error = $this->checkData(db_data::OPERATION_INSERT);
		if ($error !== db_data::ERROR_NONE) {
			return $error;
		}

		if(CheckDB::isTournament($this->tournament_id) === false) {
			return self::ERROR_TOURNAMENT_NOTFOUND;
		}

		if(CheckDB::isTeam($this->team_id) === false) {
			return self::ERROR_TEAM_NOTFOUND;
		}

		// Rule to follow before adding a team :
		//  - Can't add if the tournament is not in queue

		// Verify the tournament
		$tournament = new db_tournaments();
		$tournament->fill(array('tournament_id' => $this->tournament_id));
		if (intval($tournament->tournament_id) !== 0 && intval($tournament->status) !== db_tournaments::STATUS_QUEUE) {
			return self::ERROR_INSERTTOURNAMENT;
		}

		$table = $this->getTableName($this);
		$data_insert = db_data::generateInsert('tournament_id', 'team_id', array('subscribed' => 'now'));
		$db->sql_query('INSERT INTO ' . $table . ' ' . $data_insert);
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}

		return db_data::ERROR_NONE;
	}

	public function modify() {
		return db_data::ERROR_NOTALLOWED;
	}

	public function delete() {
		global $sys_conf, $db;

		if ($this->exists() === false) {
			return self::ERROR_NOTFOUND;
		}

		// Rule to follow before deleting a team :
		//  - Can't delete if the tournament is not in queue

		// Verify the tournament
		$tournament = new db_tournaments();
		$tournament->fill(array('tournament_id' => $this->tournament_id));
		if (intval($tournament->tournament_id) !== 0 && intval($tournament->status) !== db_tournaments::STATUS_QUEUE) {
			return self::ERROR_DELETETOURNAMENT;
		}

		$table = $this->getTableName($this);
		$db->sql_query('DELETE FROM ' . $table . ' WHERE ' . $this->createWhere($this->_primary));
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}

		return db_data::ERROR_NONE;
	}
}
?>