<?php
/***********************************************************
 *			db_tournaments.php
 *		    --------------------------
 * Created		: Wed, Feb 1, 2006
 * Copyright		: (C) LookStrike Team
 * WebSite		: http://www.lookstrike.com
 *
 * $Id: db_tournaments.php,v 1.6 2008/01/19 13:35:47 jsgoupil Exp $
 *
 ***********************************************************/
include_once($sys_conf['path']['real'] . '/modules/class/db/db_data.php');
include_once($sys_conf['path']['real'] . '/modules/class/db/db_tournament_teams.php');
include_once($sys_conf['path']['real'] . '/modules/class/db/db_matches.php');
include_once($sys_conf['path']['real'] . '/modules/class/db/db_trees.php');

class db_tournaments extends db_data {
	const ERROR_TOURNAMENT_NOTFOUND	= 1;
	const ERROR_GAME_NOTFOUND	= 2;
	const ERROR_NOTINQUEUE		= 3;

	const STATUS_QUEUE		= 0;
	const STATUS_PROGRESS		= 1;
	const STATUS_OVER		= 2;

	public $tournament_id;		// int(11)
	public $game_id;		// int(11)
	public $description;		// text
	public $price;			// decimal(5,2)
	public $minimal_player;		// tinyint(1)
	public $single_user;		// tinyint(1)
	public $status;			// tinyint(1)
	public $itselfsub;		// tinyint(1)
	public $server_group;		// int(11)
	public $inserted;		// timestamp

	protected $_ERROR_LANGUAGE = array(
		self::ERROR_TOURNAMENT_NOTFOUND	=> 'TOURNAMENT_NOTFOUND',
		self::ERROR_GAME_NOTFOUND	=> 'TOURNAMENT_GAME_NOTFOUND',
		self::ERROR_NOTINQUEUE		=> 'TOURNAMENT_NOTINQUEUE'
	);

	protected $_primary = 'tournament_id';
	protected $_unique = array();
	protected $_required_admin		= array(
		'game_id',
		'description',
	);
	protected $_required = array();
	protected $_filter = array();
	protected $_checkbox = array(
		'single_user',
		'itselfsub'
	);

	public function __construct() {
		$this->tournament_id	= 0;
		$this->game_id		= 0;
		$this->description	= '';
		$this->price		= 0.0;
		$this->minimal_player	= 0;
		$this->single_user	= 0;
		$this->status		= 0;
		$this->itselfsub	= 0;
		$this->server_group	= 0;
		$this->inserted		= date('Y-m-d H:i:s');
	}


	public function insert() {
		global $db;

		$error = $this->checkData(db_data::OPERATION_INSERT);
		if ($error !== db_data::ERROR_NONE) {
			return $error;
		}

		if(CheckDB::isGame($this->game_id) === false) {
			return self::ERROR_GAME_NOTFOUND;
		}

		$this->{$this->_primary} = 0;
		$table = $this->getTableName($this);
		$data_insert = db_data::generateInsert('game_id', 'description', 'price', 'minimal_player', 'single_user', 'status', 'itselfsub', array('server_group' => 'ifnull'), array('inserted' => 'now'));
		$db->sql_query('INSERT INTO ' . $table . ' ' . $data_insert);
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}
		$this->{$this->_primary} = $db->sql_insertid();

		return db_data::ERROR_NONE;
	}

	public function modify() {
		global $db;

		if ($this->exists() === false) {
			return self::ERROR_TOURNAMENT_NOTFOUND;
		}

		$error = $this->checkData(db_data::OPERATION_MODIFY);
		if ($error !== db_data::ERROR_NONE) {
			return $error;
		}


		if(CheckDB::isGame($this->game_id) === false) {
			return self::ERROR_GAME_NOTFOUND;
		}

		$table = $this->getTableName($this);
		$data_modify = db_data::generateModify('game_id', 'description', 'price', 'minimal_player', 'status', 'itselfsub', array('server_group' => 'ifnull'), array('inserted' => 'same'));
		$db->sql_query('UPDATE ' . $table . ' SET ' . $data_modify  . ' WHERE ' . $this->_primary . '=\'' . $this->{$this->_primary} . '\'');
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}

		return db_data::ERROR_NONE;
	}

	public function delete() {
		global $sys_conf, $db;

		if ($this->exists() === false) {
			return self::ERROR_TOURNAMENT_NOTFOUND;
		}

		// Rule to follow before deleting a team :
		//  - Can't delete if status is not in queue

		if (intval($this->status) !== self::STATUS_QUEUE) {
			return self::ERROR_NOTINQUEUE;
		}

		// We search participating teams in this tournament
		$tournaments = db_data::searchFor('tournament_teams', array('tournament_id' => $this->tournament_id));
		$c = count($tournaments);
		for ($i = 0; $i < $c; $i++) {
			$tournament_teams = new db_tournament_teams();
			$tournament_teams->fill(array('AND' => array('tournament_id' => $this->tournament_id, 'team_id' => $tournaments[$i]['team_id'])));
			$tournament_teams->delete();
		}

		// We search for matches tree
		$tree = db_data::searchForClass('db_trees', array('tournament_id' => $this->tournament_id), '', '', 'tree_id');
		$c = count($tree);
		for ($i = 0; $i < $c; $i++) {
			$tree[$i]->delete();
		}

		// We search for matches not in tree
		$matches = db_data::searchForClass('db_matches', array('tournament_id' => $this->tournament_id), '', '', 'match_id');
		$c = count($matches);
		for ($i = 0; $i < $c; $i++) {
			$matches[$i]->delete();
		}

		// We can delete the tournament
		// Can't be undone !

		$table = $this->getTableName($this);
		$db->sql_query('DELETE FROM ' . $table . ' WHERE ' . $this->_primary . '=\'' . $this->{$this->_primary} . '\'');
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}

		return db_data::ERROR_NONE;
	}
}
?>