<?php
/***********************************************************
 *			db_trees.php
 *		    -------------------------
 * Created		: Wed, Feb 1, 2006
 * Copyright		: (C) LookStrike Team
 * WebSite		: http://www.lookstrike.com
 *
 * $Id: db_trees.php,v 1.5 2008/01/27 01:58:49 jsgoupil Exp $
 *
 ***********************************************************/
include_once($sys_conf['path']['real'] . '/modules/class/db/db_data.php');
include_once($sys_conf['path']['real'] . '/modules/class/db/db_tournaments.php');
include_once($sys_conf['path']['real'] . '/modules/class/db/db_matches.php');

/**
 * If Pool :
 *  - previous = 0 or tree_id of the previous pool
 *  - tree_type = 0
 *
 * If Winning Tree
 *  - previous = 0
 *  - tree_type = depth
 *
 * If Losing Tree
 *  - previous = winning_tree
 *  - tree_type = depth
 *
 * If Final of a Tree
 *  - previous = winning_tree
 *  - tree_type = 0
 */
class db_trees extends db_data {
	const ERROR_NOTFOUND		= 1;
	const ERROR_TOURNAMENT_NOTFOUND	= 2;
	const ERROR_TREE_NOTFOUND	= 3;
	const ERROR_ADDTOURNAMENT	= 4;
	const ERROR_INCORRECTPREVIOUS	= 5;

	const TYPE_POOL = 0;

	public $tree_id;		// int(11)
	public $tournament_id;		// int(11) -> tournaments.tournament_id
	public $previous;		// int(11) -> trees.tree_id [optional]
	public $tree_type;		// tinyint(1)
	public $tie;			// tinyint(1)

	protected $_ERROR_LANGUAGE = array(
		self::ERROR_NOTFOUND		=> 'TREE_NOTFOUND',
		self::ERROR_TOURNAMENT_NOTFOUND	=> 'TOURNAMENT_NOTFOUND',
		self::ERROR_TREE_NOTFOUND	=> 'TREE_NOTFOUND',
		self::ERROR_ADDTOURNAMENT	=> 'MATCHES_ADDTOURNAMENT',
		self::ERROR_INCORRECTPREVIOUS	=> 'TREE_INCORRECTPREVIOUS'
	);

	protected $_primary = 'tree_id';
	protected $_unique = array();
	protected $_required_admin		= array(
		'tournament_id',
		'previous',
		'tree_type',
		'tie'
	);
	protected $_required = array();
	protected $_filter = array();

	public function __construct() {
		$this->tree_id		= 0;
		$this->tournament_id	= 0;
		$this->previous		= 0;
		$this->tree_type	= 0;
		$this->tie		= 0;
	}

	public function insert() {
		global $db;

		$error = $this->checkData(db_data::OPERATION_INSERT);
		if ($error !== db_data::ERROR_NONE) {
			return $error;
		}

		if(CheckDB::isTournament($this->tournament_id) === false) {
			return self::ERROR_TOURNAMENT_NOTFOUND;
		}

		if(intval($this->previous) > 0 && CheckDB::isTree($this->previous) === false) { // Optional
			return self::ERROR_TREE_NOTFOUND;
		}

		// We can't do a tree with a previous as a pool but we can do the opposite
		if(intval($this->previous) > 0 && intval($this->tree_type) > 0) {
			// Do the check
			$previous = new db_trees();
			$previous->fill(array('tree_id'=>$this->previous));
			if(intval($previous->tree_type) === 0) {
				return self::ERROR_INCORRECTPREVIOUS;
			}
		}

		// Rule to follow before adding a tree :
		//  - Can't add a match if the tournament is not in progress

		// Verify the tournament
		$tournament = new db_tournaments();
		$tournament->fill(array('tournament_id' => $this->tournament_id));
		if (intval($tournament->tournament_id) !== 0 && intval($tournament->status) !== db_tournaments::STATUS_PROGRESS) {
			return self::ERROR_ADDTOURNAMENT;
		}

		$this->{$this->_primary} = 0;
		$table = $this->getTableName($this);
		$data_insert = db_data::generateInsert('tournament_id', 'previous', 'tree_type', 'tie');
		$db->sql_query('INSERT INTO ' . $table . ' ' . $data_insert);
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}
		$this->{$this->_primary} = $db->sql_insertid();

		return db_data::ERROR_NONE;
	}

	public function modify() {
		global $db;

		if ($this->exists() === false) {
			return self::ERROR_NOTFOUND;
		}

		$error = $this->checkData(db_data::OPERATION_MODIFY);
		if ($error !== db_data::ERROR_NONE) {
			return $error;
		}

		if(CheckDB::isTournament($this->tournament_id) === false) {
			return self::ERROR_TOURNAMENT_NOTFOUND;
		}

		if(intval($this->previous) > 0 && CheckDB::isTree($this->previous) === false) { // Optional
			return self::ERROR_TREE_NOTFOUND;
		}

		$table = $this->getTableName($this);
		$data_modify = db_data::generateModify('tournament_id', 'previous', 'tree_type', 'tie');
		$db->sql_query('UPDATE ' . $table . ' SET ' . $data_modify  . ' WHERE ' . $this->_primary . '=\'' . $this->{$this->_primary} . '\'');
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}

		return db_data::ERROR_NONE;
	}

	public function delete() {
		global $sys_conf, $db;

		if ($this->exists() === false) {
			return self::ERROR_NOTFOUND;
		}

		// Rule to follow before deleting a tree :
		//  - No rules

		// Delete matches
		$matches = db_data::searchForClass('db_matches', array('tree_id' => $this->tree_id), '', '', 'match_id');
		$c = count($matches);
		for ($i = 0; $i < $c; $i++) {
			$matches[$i]->delete();
		}

		// Deletes associated trees
		$trees = db_data::searchForClass('db_trees', array('previous' => $this->tree_id), '', '', 'tree_id');
		$c = count($trees);
		for ($i = 0; $i < $c; $i++) {
			$trees[$i]->delete();
		}

		$table = $this->getTableName($this);
		$db->sql_query('DELETE FROM ' . $table . ' WHERE ' . $this->_primary . '=\'' . $this->{$this->_primary} . '\'');
		if ($db->sql_goterror() === true) {
			return db_data::ERROR_SQL;
		}

		return db_data::ERROR_NONE;
	}
}
?>