<?php
/***********************************************************
 *			DocumentGraphic.php
 *		    ---------------------------
 * Created		: Fri, Oct 22, 2004
 * Copyright		: (C) LookStrike Team
 * WebSite		: http://www.lookstrike.com
 *
 * $Id: DocumentGraphic.php,v 1.2 2008/01/24 16:34:30 jsgoupil Exp $
 *
 ***********************************************************/
if (!defined('IN_LS')) die('You are not allowed to access to this page.');
include_once($sys_conf['path']['real'] . '/modules/class/draw/Position.php');
include_once($sys_conf['path']['real'] . '/modules/class/draw/Dimension.php');
include_once($sys_conf['path']['real'] . '/modules/class/draw/Font.php');

class TDocumentGraphic {
	const DEFAULT_TITLE	= '';
	const START_POSX	= 0;
	const START_POSY	= 0;
	const SPACE_WIDTH	= 40; // Space between components
	const SPACE_HEIGHT	= 30; // Space between components
	const TITLE_FONT	= 5;
	const DEFAULT_BORDER	= true;

	protected $title;		// String
	protected $font;		// TFont or int
	protected $position;		// Position
	protected $space;		// TDimension
	protected $border;		// Boolean
	protected $components;		// implements draw method
	protected $im;			// image ressource

	public function __construct() {
		$this->setTitle(self::DEFAULT_TITLE);
		$this->setFont(self::TITLE_FONT);
		$this->setPosition(new TPosition(self::START_POSX, self::START_POSY));
		$this->setSpace(new TDimension(self::SPACE_WIDTH, self::SPACE_HEIGHT));
		$this->setBorder(self::DEFAULT_BORDER);
		$this->components = array();
	}

	public function setTitle($title) {
		$this->title = strval($title);
		if(!empty($this->title)) {
			$this->setPosition(new TPosition(0, 30)); // Let some space for the title
		}
	}

	public function setFont($a1) {
		if($a1 instanceof TFont) {
			$this->font = $a1;
		} else {
			$this->font = (int)$a1;
		}
	}

	public function setPosition(TPosition $position) {
		$this->position = $position;
	}

	public function setSpace(TDimension $space) {
		$this->space = $space;
	}

	public function setBorder($a1) {
		$this->border = (bool)$a1;
	}

	public function getTitle() {
		return $this->title;
	}

	public function getFont() {
		return $this->font;
	}

	public function getPosition() {
		return $this->position;
	}

	public function getSpace() {
		return $this->space;
	}

	public function getBorder() {
		return $this->border;
	}

	public function count() {
		return count($this->components);
	}

	public function add(Drawable $component) {
		$this->components[] = $component;
	}

	/**
	 * Gets specific component
	 */
	public function get($i) {
		if (isset($this->components[$i])) {
			return $this->components[$i];
		} else {
			return NULL;
		}
	}

	/**
	 * Removes specific componentn
	 */
	public function remove($a1) {
		if (is_int($a1)) { // int
			array_splice($this->components, $a1, 1);
		} else { // class instance
			$c = count($this->components);
			for ($i = 0; $i < $c; $i++) {
				if ($this->get($i) === $a1) {
					array_splice($this->components, $i--, 1);
				}
			}
		}
	}

	/**
	 * Removes all MatchBox
	 */
	public function removeAll() {
		$this->components = array();
	}

	public function draw() {
		$c = count($this->components);
		for($i = 0; $i < $c; $i++) {
			$this->components[$i]->draw();
		}
	}
}
?>