<?php
/***********************************************************
 *			DrawTree.php
 *		    --------------------
 * Created		: Tue, Apr 18, 2006
 * Copyright		: (C) LookStrike Team
 * WebSite		: http://www.lookstrike.com
 *
 * $Id: DrawTree.php,v 1.5 2008/02/02 01:25:30 jsgoupil Exp $
 *
 ***********************************************************/
if (!defined('IN_LS')) die('You are not allowed to access to this page.');
include_once($sys_conf['path']['real'] . '/modules/class/draw/DrawRule.php');
include_once($sys_conf['path']['real'] . '/modules/class/draw/TreePosition.php');

class TDrawTree extends TDrawRule {
	private $tree;
	private $MatchBox_default;	// TMatchBox

	public function __construct() {
		parent::__construct();
		$matchBox = new TMatchBox('');
		$matchBox->add( new TScoreBox( '', '' ) );
		$this->setDefault( $matchBox );
	}

	private function makeTree() {
		if( $this->pRule !== null && $this->pRule->GetDepth() >= 1 ) {
			$this->tree = new TTreePosition( $this->pRule->GetDepth() );
			return true;
		} else {
			return false;
		}
	}

	public function load( TRule $pRule ) {
		$c = pow( 2, $pRule->GetDepth() ) - 1;
		for( $i = 0; $i < $c; $i++ ) {
			$this->add( clone $this->getDefault(), $i );
		}

		parent::load( $pRule );
	}

	/**
	 * Draws Tree
	 */
	public function draw() {
		if( $this->makeTree() ) {
			// We must have as many components ready or more
			if( count( $this->components ) < pow( 2, $this->pRule->GetDepth() ) - 1 ) {
				return;
			}

			/* DEFINITION GRANDEUR IMAGE */
			$tree_width = $this->countWidth() + ( ( $this->pRule->GetDepth() - 1 ) * $this->getSpace()->getWidth() ) + $this->getPosition()->getPosX() + 1;
			$tree_height = $this->countHeight() + ( ( pow( 2, $this->pRule->GetDepth() - 1 ) - 1 ) * $this->getSpace()->getHeight() ) + $this->getPosition()->getPosY() + 1;
			$this->im = imagecreatetruecolor( $tree_width, $tree_height );

			$colorWhite = new TColor( 'white' );
			imagefill( $this->im, 0, 0, $colorWhite->allocate( $this->im ) );

			/* AFFICHAGE DES INFORMATIONS */
			$posX = $this->getPosition()->getPosX();
			$pos = array();
			$saved_height = array();
			for($i = $this->pRule->GetDepth(); $i > 0; $i--) {
				$pos[] = $this->tree->getPosByDepth( $i );
			}

			$c = count($pos);
			for( $i = 0; $i < $c; $i++ ) {
				// This if structure is used to find the X position
				if( $i > 0 ) {
					$maxWidth = 0;
					$c2 = count( $pos[ $i - 1 ] );
					for( $j = 0; $j < $c2; $j++ ) { // We search for the maximal width before our current level
						$currentWidth = $this->getCurrentWidth( $pos[ $i - 1 ][ $j ] );
						if( $currentWidth > $maxWidth ) {
							$maxWidth = $currentWidth;
						}
					}
					$posX += $maxWidth + $this->getSpace()->getWidth();
				}
				$posY = $this->getPosition()->getPosY();
				$c2 = count( $pos[$i] );
				for( $j = 0; $j < $c2; $j++ ) {
					if( $i > 0 ) {
						// Link
						$this->get( $pos[ $i - 1 ][ $j * 2 ] )->createLink( $this->get( $pos[ $i ][ $j ] ), true );
						$this->get( $pos[ $i - 1 ][ $j * 2 + 1 ] )->createLink( $this->get( $pos[ $i ][ $j ] ), true );
						// Taille total = taillebloc1 + taillebloc2 + space (entre chaque bloc)
						$taillebloc1 = $this->getCurrentHeight( $pos[ $i -1 ][ $j * 2 ] );
						$taillebloc2 = $this->getCurrentHeight( $pos[ $i - 1 ][ $j * 2 + 1]);
						$space = $saved_array[ $j * 2 + 1 ] - ( $saved_array[ $j * 2 ] + $taillebloc1 );
						$height_total = $taillebloc1 + $taillebloc2 + $space;
						$posY = $saved_array[ $j * 2 ] + $height_total / 2 - $this->getCurrentHeight( $pos[ $i ][ $j ]) / 2;
					} elseif( $j > 0 ) {
						$height = $this->getCurrentHeight( $pos[ $i ][ $j - 1 ] );
						$posY += $this->getSpace()->getHeight() + $height;
					}

					$saved_array[ $j ] = $posY;
					$this->get( $pos[ $i ][ $j ] )->setPosition( new TPosition( $posX, $posY ) );
					if($this->strBackground !== NULL) {
						$this->get( $pos[ $i ][ $j ] )->getTitle()->setBackgroundImage( $this->strBackground );
					}
				}
			}

			// Draw all
			$c = pow( 2, $this->pRule->GetDepth() ) - 1;
			for( $i = 0; $i < $c; $i++ ) {
				$this->get( $i )->draw( $this->im );
			}

			/* AJOUT SUPERFICIEL */
			$this->drawTitle( $this->im, $tree_width );
			$this->drawBorder( $this->im, $tree_width, $tree_height );

			return $this->im;
		} else {
			return false;
		}
	}

	/**
	 * We check for each column the bigger width you can found and we add
	 * it to our current size we have found (with the previous column)...
	 */
	protected function countWidth() {
		$width = 0;
		for( $i = 1; $i <= $this->pRule->GetDepth(); $i++ ) {
			$pos = $this->tree->getPosByDepth( $i );
			$maxWidth = 0;
			$c = count($pos);
			for( $j = 0; $j < $c; $j++ ) {
				$currentWidth = $this->getCurrentWidth( $pos[ $j ] );
				if($currentWidth > $maxWidth) {
					$maxWidth = $currentWidth;
				}
			}
			$width += $maxWidth;
		}
		return $width;
	}

	/**
	 * The max height is the height of each box the most of the left...
	 */
	protected function countHeight() {
		$height = 0;
		$pos = $this->tree->getPosByDepth( $this->pRule->GetDepth() );
		$c = count($pos);
		for( $i = 0; $i < $c; $i++ ) {
			$height += $this->getCurrentHeight( $pos[ $i ] );
		}
		return $height;
	}

	private function getCurrentWidth( $a1 ){
		$MatchBox = $this->get( $a1 );
		return ( $MatchBox === NULL ) ? $this->getDefault()->getMaxWidth() : $MatchBox->getMaxWidth();
	}

	private function getCurrentHeight( $a1 ){
		$MatchBox = $this->get( $a1 );
		return ( $MatchBox === NULL ) ? $this->getDefault()->getMaxHeight() : $MatchBox->getMaxHeight();
	}

	public function setDefault( TMatchBox $a1 ){
		$this->MatchBox_default = $a1;
	}

	public function getDefault(){
		return $this->MatchBox_default;
	}

	public function add( Drawable $component ) {
		$nPosition = intval( func_get_arg( 1 ) );
		if( $nPosition >= 0 ) {
			$this->components[ $nPosition ] = $component;
		}
	}

	public function remove($a1) {
		if ( is_int( $a1 ) ) { // int
			array_splice( $this->components, $a1, 1 );
		} else { // class instance
			foreach( $this->components as $key => $val ) {
				if( $a1 === $val ) {
					unset($this->components[$key]); // To keep the index
				}
			}
		}
	}
};
?>