<?php
/***********************************************************
 *			Gradient.php
 *		    -------------------
 * Created		: Sat, Oct 23, 2004
 * Copyright		: (C) LookStrike Team
 * WebSite		: http://www.lookstrike.com
 *
 * $Id: Gradient.php,v 1.2 2008/01/24 16:34:30 jsgoupil Exp $
 *
 ***********************************************************/
if (!defined('IN_LS')) die('You are not allowed to access to this page.');
include_once($sys_conf['path']['real'] . '/modules/class/draw/Color.php');
include_once($sys_conf['path']['real'] . '/modules/class/draw/Position.php');
include_once($sys_conf['path']['real'] . '/modules/class/draw/Dimension.php');
include_once($sys_conf['path']['real'] . '/modules/class/draw/interface.php');

class TGradient implements Drawable {
	const DEGRADE_HORIZONTAL	= 1;
	const DEGRADE_VERTICAL		= 2;

	private $position = NULL;		// TPosition
	private $dimension = NULL;		// TDimension
	private $color = array();		// TColor[]
	private $direction;			// int (constant)

	public function __construct(TPosition $pos, TDimension $dim, $color, $direction) {
		$this->setPosition($pos);
		$this->setDimension($dim);
		$this->setColor($color);
		$this->setDirection($direction);
	}

	public function setPosition(TPosition $a1) {
		$this->position = $a1;
	}

	public function setDimension(TDimension $a1) {
		$this->dimension = $a1;
	}

	public function setColor($a1) {
		$this->color = $a1;
	}

	public function setDirection($a1) {
		if ($a1 > 2 || $a1 < 1) {
			$this->direction = self::DEGRADE_HORIZONTAL;
		} else {
			$this->direction = intval($a1);
		}
	}

	public function getPosition() {
		return $this->position;
	}

	public function getDimension() {
		return $this->dimension;
	}

	public function getColor() {
		return $this->color;
	}

	public function getDirection() {
		return $this->direction;
	}

	private function getColorAt($a1) {
		return $this->color[$a1];
	}

	public function draw($im) {
		$ret = NULL;
		if (count($this->getColor()) === 1) {
			// This is not a gradient !
		} else {
			if ($this->getDirection() === self::DEGRADE_HORIZONTAL) {
				$ret = $this->drawHorizontal($im);
			} elseif ($this->getDirection() === self::DEGRADE_VERTICAL) {
				$ret = $this->drawVertical($im);
			}
		}
		return $ret;
	}

	private function drawHorizontal($im) {
		$w = $this->getDimension()->getWidth();
		$n = $w / (count($this->getColor()) - 1);
		$currentColor = new TColor(0, 0, 0);
		$currentI = 0;
		$lastColor = -1.0;
		for ($i = 0; $i < $w; $i++, $currentI++) {
			$x = $this->getPosition()->getPosX();
			$y = $this->getPosition()->getPosY();
			$h = $this->getDimension()->getHeight();
			$iColor = floor($i / $n);
			if ($lastColor !== $iColor) {
				$currentI = 0;
				$lastColor = $iColor;
			}
			$c1 = array($this->getColorAt($iColor)->r(), $this->getColorAt($iColor)->g(), $this->getColorAt($iColor)->b());
			$c2 = array($this->getColorAt($iColor+1)->r(), $this->getColorAt($iColor+1)->g(), $this->getColorAt($iColor+1)->b());
			for ($j = 0; $j < 3; $j++) {
				$c1[$j] = $c1[$j] + ($currentI / $n) * ($c2[$j] - $c1[$j]);
			}
			$currentColor->setColor($c1[0], $c1[1], $c1[2]);
			imageline($im, $x + $i, $y, $x + $i, $y + $h, $currentColor->allocate($im));
		}
	}

	private function drawVertical($im) {
		$h = $this->getDimension()->getHeight();
		$n = $h / (count($this->getColor()) - 1);
		$currentColor = new TColor(0, 0, 0);
		$c1 = array();
		$c2 = array();
		$currentI = 0;
		$lastColor = -1.0;
		for ($i = 0; $i < $h; $i++, $currentI++) {
			$x = $this->getPosition()->getPosX();
			$y = $this->getPosition()->getPosY();
			$w = $this->getDimension()->getWidth();
			$iColor = floor($i / $n);
			if ($lastColor !== $iColor) {
				$currentI = 0;
				$lastColor = $iColor;
			}
			$c1 = array($this->getColorAt($iColor)->r(), $this->getColorAt($iColor)->g(), $this->getColorAt($iColor)->b());
			$c2 = array($this->getColorAt($iColor+1)->r(), $this->getColorAt($iColor+1)->g(), $this->getColorAt($iColor+1)->b());
			for ($j = 0; $j < 3; $j++) {
				$c1[$j] = $c1[$j] + ($currentI / $n) * ($c2[$j] - $c1[$j]);
			}
			$currentColor->setColor($c1[0], $c1[1], $c1[2]);
			imageline($im, $x, $y + $i, $x + $w, $y + $i, $currentColor->allocate($im));
		}
	}
};
?>