<?php
/***********************************************************
 *			MatchBox.php
 *		    --------------------
 * Created		: Tue, Oct 19, 2004
 * Copyright		: (C) LookStrike Team
 * WebSite		: http://www.lookstrike.com
 *
 * $Id: MatchBox.php,v 1.4 2008/01/24 16:34:30 jsgoupil Exp $
 *
 ***********************************************************/
if (!defined('IN_LS')) die('You are not allowed to access to this page.');
include_once($sys_conf['path']['real'] . '/modules/class/draw/ScoreBox.php');
include_once($sys_conf['path']['real'] . '/modules/class/draw/Winner.php');

class TMatchBox implements Drawable {
	private $id;			// Int : Optional
	private $title;			// TTextBox
	private $ScoreBox = array();	// TScoreBox[]
	private $position;		// TPosition

	private $linkto = array();	// TMatchBox
	private $linkto_attach = array();// Boolean

	public function __construct($a1) {
		$this->setTitle($a1);
		$this->setPosition(new TPosition(0, 0));
	}

	public function setId($a1) {
		$this->id = (int)$a1;
	}

	public function setTitle($a1) {
		if ($a1 instanceof TTextBox) {
			$this->title = $a1;
		} else {
			$this->title = new TTextBox($a1);
			$this->setDefault($this->title);
		}
	}

	public function setPosition(TPosition $a1) {
		$this->position = $a1;
	}

	/**
	 * No getter since we set on many different controls.
	 */
	public function setBorderColor(TColor $a1) {
		$c = $this->count();
		for ($i = 0; $i < $c; $i++) {
			$this->get($i)->setBorderColor($a1);
		}
	}

	public function getId() {
		return $this->id;
	}

	public function getTitle() {
		if ($this->title === NULL) {
			return new TTextBox('');
		} else {
			return $this->title;
		}
	}

	public function getPosition() {
		return $this->position;
	}

	public function count() {
		return count($this->ScoreBox);
	}

	public function add(TScoreBox $a1) {
		$this->ScoreBox[] = $a1;
	}

	public function get($i) {
		if (isset($this->ScoreBox[$i])) {
			return $this->ScoreBox[$i];
		} else {
			return NULL;
		}
	}

	public function remove($a1) {
		if (is_int($a1)) {
			array_splice($this->ScoreBox, $a1, 1);
		} elseif ($a1 instanceof TScoreBox) {
			for ($i = 0; $i < $this->count(); $i++) {
				if ($this->get($i) === $a1) {
					array_splice($this->ScoreBox, $i--, 1);
				}
			}
		}
	}

	public function removeAll() {
		while ($this->count() > 0) {
			$this->remove(0);
		}
	}

	public function draw($im) {
		$dimension = clone $this->getTitle()->getDimension();
		$former_title_position = clone $this->getTitle()->getPosition();
		$this->getTitle()->getPosition()->setPosX($former_title_position->getPosX() + $this->position->getPosX());
		$this->getTitle()->getPosition()->setPosY($former_title_position->getPosY() + $this->position->getPosY());
		$this->getTitle()->draw($im);
		$lastPos = new TPosition($this->getTitle()->getPosition()->getPosX(), $this->getTitle()->getPosition()->getPosY() + $this->getTitle()->getDimension()->getHeight());
		$c = $this->count();
		for ($i = 0; $i < $c; $i++) {
			$former_box_position = clone $this->get($i)->getPosition();
			$this->get($i)->setPosition($lastPos);
			$this->get($i)->draw($im);
			$lastPos = new TPosition($this->get($i)->getPosition()->getPosX(), $this->get($i)->getPosition()->getPosY() + $dimension->getHeight());
			$this->get($i)->setPosition($former_box_position);
		}
		$this->makeLink($im);
		$this->getTitle()->setPosition($former_title_position);
	}

	public function setWinner(ColorMatchBox $callback) {
		$call = new $callback();
		$winner = $this->searchWinner();
		$this->unsetWinner($callback);
		$c = $this->count();
		for ($i = 0; $i < $c; $i++) {
			if (in_array($i, $winner)) {
				$call->doWin($this->get($i));
			} elseif ($this->get($i) !== NULL && $this->get($i)->getTeam()->getText() !== '' && $this->matchPlayed()) {
				$call->doLost($this->get($i));
			}
		}
	}

	public function unsetWinner(ColorMatchBox $callback) {
		$call = new $callback();
		$c = $this->count();
		for ($i = 0; $i < $c; $i++) {
			$call->doClear($this->get($i));
		}
	}

	public function getWinner() {
		$winner = $this->searchWinner();
		if (count($winner) >= 1) {
			return $this->get($winner[0]);
		} else {
			return NULL;
		}
	}

	public function matchPlayed() {
		$match_played = false;
		$c = $this->count();
		for ($i = 0; $i < $c; $i++) {
			if (intval($this->get($i)->getScore()->getText()) !== 0) {
				$match_played = true;
				break;
			}
		}
		return $match_played;
	}

	private function searchWinner() {
		$maxScore = 0;
		$winner = array();
		$c = $this->count();
		for ($i = 0; $i < $c; $i++) {
			if (intval($this->get($i)->getScore()->getText()) > 0) {
				if (intval($this->get($i)->getScore()->getText()) > $maxScore) {
					$maxScore = intval($this->get($i)->getScore()->getText());
					unset($winner);
					$winner[] = $i;
				} elseif (intval($this->get($i)->getScore()->getText()) === $maxScore) {
					$winner[] = $i;
				}
			}
		}
		return $winner;
	}

	public function createLink(TMatchBox $matchbox, $attach = true) {
		$this->linkto[] = $matchbox;
		$this->linkto_attach[] = (bool)$attach;
	}

	public function getMaxHeight() {
		$height = $this->getTitle()->getDimension()->getHeight();
		$c = $this->count();
		for ($i = 0; $i < $c; $i++) {
			$height += $this->get($i)->getTeam()->getDimension()->getHeight();
		}
		return $height;
	}

	public function getMaxWidth() {
		return $this->getTitle()->getDimension()->getWidth();
	}

	public function getMiddleTeam() {
		return ($this->getMaxHeight() - $this->getTitle()->getDimension()->getHeight()) / 2 + $this->getTitle()->getDimension()->getHeight();
	}

	private function setDefault(TTextBox $a1) {
		$a1->setPosition(new TPosition(0, 0));
		$a1->setDimension(new TDimension(150, 20));
		$a1->setBorderColor(new TColor(0, 0, 0));
		$a1->setForegroundColor(new TColor(255, 255, 255));
		$a1->setBackgroundColor(new TColor(0, 0, 0));
		$a1->setFont(5);
	}

	private function makeLink($im) {
		$c = count($this->linkto);
		for ($i = 0; $i < $c; $i++) {
			imageline(
				$im,
				$this->getPosition()->getPosX() + $this->getMaxWidth() + (($this->linkto_attach[$i] === true) ? 5 : 0),
				$this->getPosition()->getPosY() + $this->getMiddleTeam(),
				$this->linkto[$i]->getPosition()->getPosX(),
				$this->linkto[$i]->getPosition()->getPosY() + $this->linkto[$i]->getMiddleTeam(),
				$this->getTitle()->getBackgroundColor()->allocate($im)
			);

			if ($this->linkto_attach[$i] === true) {
				imagefilledellipse(
					$im,
					$this->getPosition()->getPosX() + $this->getMaxWidth() + 5,
					$this->getPosition()->getPosY() + $this->getMiddleTeam(),
					10,
					10,
					$this->getTitle()->getBackgroundColor()->allocate($im)
				);
			}
		}
	}
};
?>