<?php
/***********************************************************
 *			TextBox.php
 *		    -------------------
 * Created		: Tue, Oct 19, 2004
 * Copyright		: (C) LookStrike Team
 * WebSite		: http://www.lookstrike.com
 *
 * $Id: TextBox.php,v 1.3 2008/01/24 16:34:30 jsgoupil Exp $
 *
 ***********************************************************/
if (!defined('IN_LS')) die('You are not allowed to access to this page.');
include_once($sys_conf['path']['real'] . '/modules/class/draw/Box.php');
include_once($sys_conf['path']['real'] . '/modules/class/draw/Font.php');
include_once($sys_conf['path']['real'] . '/modules/class/draw/interface.php');

class TTextBox extends TBox implements Drawable {
	const PADDING_LEFT = 5;
	const PADDING_TOP = 3;

	private $text = NULL;
	private $font = 1;		 // TFont or integer
	private $colorForeground = NULL; // TColor
	private $colorBackground = NULL; // TColor
	private $imageBackground = NULL; // string (path)

	public function __construct($a1) {
		$this->setText($a1);
	}

	public function setText($a1) {
		$this->text = strval($a1);
	}

	public function setFont($a1) {
		if($a1 instanceof TFont) {
			$this->font = $a1;
		} else {
			$this->font = (int)$a1;
		}
	}

	public function setForegroundColor(TColor $a1) {
		$this->colorForeground = $a1;
	}

	public function setBackgroundColor(TColor $a1) {
		$this->colorBackground = $a1;
	}

	public function setBackgroundImage($a1) {
		$this->imageBackground = $a1;
	}

	public function getText() {
		return $this->text;
	}

	public function getFont() {
		return $this->font;
	}

	public function getForegroundColor() {
		if ($this->colorForeground === NULL) {
			return new TColor(0, 0, 0);
		} else {
			return $this->colorForeground;
		}
	}

	public function getBackgroundColor() {
		if ($this->colorBackground === NULL) {
			return new TColor(255, 255, 255);
		} else {
			return $this->colorBackground;
		}
	}

	public function getBackgroundImage() {
		return $this->imageBackground;
	}

	public function getDimension() {
		if ($this->dimension === NULL) {
			if($this->font instanceof TFont) {
				$f = clone $this->getFont();
				$f->setText($this->getText());
				return new TDimension($f->getWidth() + self::PADDING_LEFT * 2, $f->getHeight() + self::PADDING_TOP * 2);
			} else {
				return new TDimension(imagefontwidth((int)$this->getFont()) * strlen($this->getText()) + self::PADDING_LEFT * 2, imagefontheight((int)$this->getFont()) + self::PADDING_TOP * 2);
			}
		} else {
			return $this->dimension;
		}
	}

	public function draw($im) {
		imagefilledrectangle(
			$im,
			$this->getPosition()->getPosX(),
			$this->getPosition()->getPosY(),
			$this->getPosition()->getPosX() + $this->getDimension()->getWidth(),
			$this->getPosition()->getPosY() + $this->getDimension()->getHeight(),
			$this->getBackgroundColor()->allocate($im)
		);
		parent::draw($im);

		if (($img = $this->getBackgroundImage()) !== NULL) {
			$imBack = imagecreatefrompng($img);
			imagecopyresized(
				$im,
				$imBack,
				$this->getPosition()->getPosX(),
				$this->getPosition()->getPosY(),
				0,
				0,
				$this->getDimension()->getWidth() + 1,
				$this->getDimension()->getHeight(),
				imagesx($imBack),
				imagesy($imBack)
			);
		}

		if($this->font instanceof TFont) {
			$this->font->setText($this->getText());
			$this->font->draw($im, $this->getForegroundColor()->allocate($im), $this->getPosition()->getPosX() + self::PADDING_LEFT, $this->getPosition()->getPosY() + self::PADDING_TOP);
		} else {
			imagestring(
				$im,
				$this->getFont(),
				$this->getPosition()->getPosX() + self::PADDING_LEFT,
				$this->getPosition()->getPosY() + self::PADDING_TOP,
				$this->getText(),
				$this->getForegroundColor()->allocate($im)
			);
		}
	}
}
?>