<?php
/***********************************************************
 *			TreePosition.php
 *		    ------------------------
 * Created		: Fri, Feb 1st, 2008
 * Copyright		: (C) LookStrike Team
 * WebSite		: http://www.lookstrike.com
 *
 * $Id: TreePosition.php,v 1.1 2008/02/02 01:25:30 jsgoupil Exp $
 *
 ***********************************************************/
if (!defined('IN_LS')) die('You are not allowed to access to this page.');
include_once($sys_conf['path']['real'] . '/modules/class/draw/TreeNode.php');

class TTreePosition {
	private $nDepth;
	private $firstNode;
	private $positionArray;

	public function __construct( $nDepth ) {
		$this->setDepth( $nDepth );
	}

	public function setDepth( $nDepth ) {
		$bRedo = false;
		if( intval( $nDepth ) !== intval( $this->nDepth ) ) {
			$bRedo = true;
		}

		// TODO Set the maximum depth we can make !!
		$this->nDepth = (int)$nDepth;

		if( $bRedo ) {
			$this->nodeInit();
			$this->calculatePosArray();
		}
	}

	public function getDepth() {
		return $this->nDepth;
	}

	/**
	 * Returns the TTreeNode firstNode of the tree.
	 *
	 * @return TTreeNode
	 */
	public function getFirstNode() {
		return $this->firstNode;
	}

	/**
	 * Returns an array of position of all TTreeNode depending
	 * on the depth $nDepth.
	 *
	 * @param int $nDepth
	 * @return int[]
	 */
	public function getPosByDepth( $nDepth ) {
		if( $nDepth >= 1 && $nDepth <= $this->getDepth() ) {
			return $this->positionArray[ $nDepth ];
		} else {
			return array();
		}
	}

	private function nodeInit() {
		$this->temp_val1 = 0;
		$this->firstNode = new TTreeNode();
		return $this->_nodeInit( $this->firstNode, 0 );
	}

	private function _nodeInit( TTreeNode $currentNode, $currentPosition ) {
		$this->temp_val1++;

		if( $this->temp_val1 === $this->getDepth() ) {
			$this->temp_val1--;
			return $currentPosition;
		}

		$currentPosition++;
		$currentNode->left = new TTreeNode();
		$currentPosition = $this->_nodeInit( $currentNode->left, $currentPosition );
		$currentPosition++;
		$currentNode->right = new TTreeNode();
		$currentPosition = $this->_nodeInit( $currentNode->right, $currentPosition );

		$this->temp_val1--;
		return $currentPosition;
	}

	/**
	 * We save in cache all the position in an array to time to calculate
	 */
	private function calculatePosArray() {
		for( $i = 1; $i <= $this->getDepth(); $i++ ) {
			$this->positionArray[ $i ] = array();
			$this->temp_val1 = 0; // Deep Counter
			$this->temp_val2 = 0; // Node Counter
			$this->scanTree( $this->firstNode, $i, $this->positionArray[ $i ] );
		}
	}

	/**
	 * Scans the TreeNode and fills the $pos array
	 *
	 * @param TTreeNode $currentNode
	 * @param int $search_pos
	 * @param int[] $pos
	 */
	private function scanTree( TTreeNode $currentNode, $search_pos, &$pos ) {
		$this->temp_val1++;

		if( $this->temp_val1 === $search_pos ) {
			$pos[] = $this->temp_val2;
		}

		$this->temp_val2++;

		// We are at the end of the tree
		if( $currentNode->left === NULL || $currentNode->right === NULL ) {
			$this->temp_val1--;
			return;
		}
		// Scan left
		if( $currentNode->left !== NULL ) {
			$this->scanTree( $currentNode->left, $search_pos, $pos );
		}
		// Scan right
		if( $currentNode->right !== NULL ) {
			$this->scanTree( $currentNode->right, $search_pos, $pos );
		}
		$this->temp_val1--;
	}
}