<?php
/***********************************************************
 *			Match.php
 *		    -----------------
 * Created		: Thu, Mar 9, 2006
 * Copyright		: (C) LookStrike Team
 * WebSite		: http://www.lookstrike.com
 *
 * $Id: Match.php,v 1.2 2007/02/15 15:04:41 jsgoupil Exp $
 *
 ***********************************************************/
include_once($sys_conf['path']['real'] . '/modules/class/tournament/MatchTeam.php');
include_once($sys_conf['path']['real'] . '/modules/class/db/db_match_teams.php');
include_once($sys_conf['path']['real'] . '/modules/class/db/db_matches.php');

interface IMatchListener {
	public function OnScoreModified( TMatch $pCaller, TMatchTeam $pMatchTeam );
	public function OnEndMatch( TMatch $pCaller );
}

class TMatch {
	const TIMESTAMP_MATCHPROGRESS = '0000-00-00 00:00:00';

	private $pMatchListener;
	private $pMatch;
	private $pMatchTeams = array();

	public function __construct( $pMatchListener = NULL ) {
		$this->pMatchListener = $pMatchListener;
	}

	/**
	 * Loads a match.
	 * $pMatch can be a db_matches or an integer.
	 * Returns true on success
	 *
	 * @param mixed $pMatch
	 * @return bool
	 */
	public function Load( $pMatch ) {
		if( $pMatch instanceof db_matches ) {
			$this->pMatch = $pMatch;
		} else {
			$this->pMatch = new db_matches();
			$this->pMatch->fill( array( $this->pMatch->getPrimary() => $pMatch ) );
		}

		if( !$this->pMatch->IsLoaded() ) {
			return false;
		}

		return true;
	}

	/**
	 * Throw an exception is the match is not loaded
	 */
	public function Loaded() {
		if( $this->pMatch === NULL || !$this->pMatch->IsLoaded() ) {
			throw new Exception('Match is not loaded.');
		}
	}

	/**
	 * Builds MatchTeams into the class.
	 *
	 * @param IMatchTeamListener $pMatchTeamListener
	 */
	public function Build( $pMatchTeamListener = NULL ) {
		$this->Loaded();

		$match_team = new db_match_teams();
		$match_teams = db_data::searchForClass( 'db_match_teams', array( 'match_id' => $this->pMatch->match_id ), '', '', $match_team->getPrimary() );

		$c = count( $match_teams );
		for( $i = 0; $i < $c; $i++ ) {
			$oMatchTeam = new TMatchTeam( $pMatchTeamListener );
			$oMatchTeam->Load( $match_teams[$i] );
			$oMatchTeam->Build();
			$this->AttachMatchTeam( $oMatchTeam );
		}
	}

	/**
	 * Returns the match.
	 *
	 * @return db_teams
	 */
	public function Get() {
		return $this->pMatch;
	}

	public function AttachMatchTeam( TMatchTeam $pMatchTeam ) {
		$this->pMatchTeams[] = $pMatchTeam;
	}

	/**
	 * Enters Score for a team.
	 *
	 * @param int $nTeamId
	 * @param int $nScore
	 */
	public function EnterScore( $nTeamId, $nScore ) {
		$nTeamId = (int)$nTeamId;
		$nScore = (int)$nScore;

		if( $pMatchTeam = $this->SearchMatchTeam( $nTeamId ) !== NULL ) {
			$pMatchTeam->Get()->score = $nScore;
			$pMatchTeam->Get()->modify();

			if( $this->pMatchListener instanceof IMatchListener ) {
				$this->pMatchListener->OnScoreModified( $this, $pMatchTeam );
			}

			return true;
		}

		return false;
	}

	public function EndMatch() {
		$this->Loaded();

		$this->pMatch->end_time = db_data::GetNow( DB_TIME_TIMESTAMP );
		$this->pMatch->modify();

		if( $this->pMatchListener instanceof IMatchListener ) {
			$this->pMatchListener->OnEndMatch( $this );
		}
	}

	private function SearchMatchTeam( $nTeamId ) {
		$c = count( $this->pMatchTeams );
		for( $i = 0; $i < $c; $i++ ) {
			if( intval( $this->pMatchTeams[$i]->Get()->team_id ) === $nTeamId ) {
				return $this->pMatchTeams[$i];
			}
		}
		return NULL;
	}

	public function _Debug( $code ) {
		$strFirst = str_repeat( '>', $code );

		echo $strFirst . 'Match.id = ' . $this->pMatch->match_id . '<br />';

		$c = count( $this->pMatchTeams );
		for( $i = 0; $i < $c; $i++ ) {
			$this->pMatchTeams[$i]->_Debug( $code + 1 );
		}
	}

	public function getMatch() {
		$this->Loaded();

		return $this->pMatch;
	}

	public function getMatchTeams() {
		return $this->pMatchTeams;
	}
};
?>