<?php
/***********************************************************
 *			Rule.php
 *		    ----------------
 * Created		: Sat, Mar 11, 2006
 * Copyright		: (C) LookStrike Team
 * WebSite		: http://www.lookstrike.com
 *
 * $Id: Rule.php,v 1.6 2008/02/02 01:25:30 jsgoupil Exp $
 *
 ***********************************************************/
include_once($sys_conf['path']['real'] . '/modules/class/db/db_matches.php');
include_once($sys_conf['path']['real'] . '/modules/class/db/db_teams.php');
include_once($sys_conf['path']['real'] . '/modules/class/db/db_tournament_teams.php');
include_once($sys_conf['path']['real'] . '/modules/class/tournament/Tournament.php');
include_once($sys_conf['path']['real'] . '/modules/class/tournament/Tree.php');
include_once($sys_conf['path']['real'] . '/modules/class/tournament/Match.php');
include_once($sys_conf['path']['real'] . '/modules/class/tournament/MatchTeam.php');

abstract class TRule implements ITournamentListener, IMatchListener, IMatchTeamListener {
	protected $pTree;
	protected $linkUp; // First (Winning)
	protected $linkDown; // Next (Losing)
	protected $bFake; // If set to true, nothing is done on the DB when creating
	protected $nRandom; // Random Number Used

	public function __construct() {
		$this->bFake = true;
		$this->nRandom = NULL;
	}

	protected abstract function _CreateRule( $nTournament, $nPrevious );
	protected abstract function _CreateMatch( $tournament_teams, $i );

	/**
	 * Saves the match
	 *
	 * @param int $nMatchId
	 * @param int[] $arrTeams
	 * @param int[] $arrScore
	 * @return bool
	 */
	public function SaveMatch( $nMatchId, $arrTeams, $arrScores ) {
		if( count( $arrTeams ) !== count( $arrScores ) ) return false;

		$match = new db_matches();
		$match->fill( array( 'match_id' => $nMatchId ) );
		if( intval( $match->match_id ) !== 0 ) {
			$db_match_teams = new db_match_teams();
			$match_teams = db_data::searchForClass( 'db_match_teams', array( 'match_id' => $nMatchId ), '', '', $db_match_teams->GetPrimary() );
			$c1 = count( $arrTeams );
			$c2 = count( $match_teams );
			$arrAlreadyInserted = array();
			$arrToSave = array(); // i=>array(team_id, score, already_inserted)
			// check if we have already inserted or not
			for( $i = 0; $i < $c1; $i++ ) {
				$bFound = false;
				for( $j = 0; $j < $c2; $j++ ) {
					if( $arrTeams[ $i ] == $match_teams[ $j ]->team_id ) {
						$bFound = true;
						$arrAlreadyInserted[] = $match_teams[ $j ];
						$arrToSave[] = array( $arrTeams[ $i ], $arrScores[ $i ], $match_teams[ $j ] );
						break;
					}
				}
				if( !$bFound ) {
					$arrToSave[] = array( $arrTeams[ $i ], $arrScores[ $i ], NULL );
				}
			}
			// Delete the not already
			for( $i = 0; $i < $c2; $i++ ) {
				if( !in_array( $match_teams[ $i ], $arrAlreadyInserted ) ) {
					$match_teams[ $i ]->delete();
				}
			}
			// Insert or modify
			$c = count( $arrToSave );
			for( $i = 0; $i < $c; $i++ ) {
				if( $arrToSave[ $i ][ 2 ] === NULL ) {
					$match_team = new db_match_teams();
					$match_team->match_id = $nMatchId;
					$match_team->team_id = $arrToSave[ $i ][ 0 ];
					$match_team->score = $arrToSave[ $i ][ 1 ];
					$match_team->insert();
				} else {
					$arrToSave[ $i ][ 2 ]->score = $arrToSave[ $i ][ 1 ];
					$arrToSave[ $i ][ 2 ]->modify();
				}
			}
			$match->end_time = db_data::GetNow( db_data::DB_TIME_TIMESTAMP );
			$match->modify();
			return true;
		}
		return false;
	}

	/**
	 * Creates the Pool/Tree and its matches
	 * Returns an array with [0]->db_data, [1]->error_number on error
	 * Or returns int random number used on no error.
	 *
	 * @param int $nTournament
	 * @param int $nPrevious
	 * @return int Random number used
	 */
	public function Create( $nTournament, $nPrevious ) {
		// Create Pool/Tree
		$error = $this->_CreateRule( $nTournament, $nPrevious );
		if( $error !== db_data::ERROR_NONE ) {
			return array( new db_trees(), $error );
		}

		// Random Number
		if( $this->nRandom === NULL ) {
			$nRandom = mt_rand();
		} else {
			$nRandom = $this->nRandom;
		}
		$strRandom = 'RAND(' . $nRandom . ')';

		// Get All Teams in the tournament
		$tournament_team = new db_tournament_teams();
		$tournament_teams = db_data::searchForClass( 'db_tournament_teams', array( 'tournament_id' => $nTournament ), $strRandom, '', $tournament_team->GetPrimary() );

		// Attach Teams
		$c = count( $tournament_teams );
		for( $i = 0; $i < $c; $i++ ) {
			$oTournamentTeam = new TTournamentTeam( NULL );
			$oTournamentTeam->Load( $tournament_teams[$i] );
			$this->pTree->AttachTournamentTeam( $oTournamentTeam );
		}

		// Create Matches
		// Before inserting, we check if its odd and if the last one already got an autowin
		$this->CheckAutowin( $tournament_teams );

		// Same $c
		for( $i = 0; $i < $c; $i += 2 ) {
			// Create Match
			$error = $this->_CreateMatch( $tournament_teams, $i );
			if( $error[1] !== db_data::ERROR_NONE ) {
				return array( new db_matches(), $error );
			}
			$match = $error[0];

			$oMatch = new TMatch( NULL );
			$oMatch->Load( $match );
			// We build dynamically with the following
			$this->pTree->AttachMatch( $oMatch );

			// Match Teams
			$match_teams = NULL;
			for( $j = 0; $j < 2; $j++ ) {
				if( isset( $tournament_teams[ $i + $j ] ) ) {
					$match_teams = new db_match_teams();
					$match_teams->match_id = $match->match_id;
					$match_teams->team_id = $tournament_teams[ $i + $j ]->team_id;
					if( !$this->bFake ) {
						$error = $match_teams->insert();
					} else {
						$error = db_data::ERROR_NONE;
					}
					if( $error !== db_data::ERROR_NONE ) {
						return array( $match_teams, $error );
					}
					$oMatchTeam = new TMatchTeam( NULL );
					$oMatchTeam->Load( $match_teams );
					// We don't Build...
					$oMatch->AttachMatchTeam( $oMatchTeam );
				} else {
					// Autowin for the PREVIOUS entered
					$match_teams->score = 1;
					if( !$this->bFake ) {
						$error = $match_teams->modify();
					} else {
						$error = db_data::ERROR_NONE;
					}
					if( $error !== db_data::ERROR_NONE ) {
						return array( $match_teams, $error );
					}
					$match->end_time = db_data::GetNow( db_data::DB_TIME_TIMESTAMP );
					if( !$this->bFake ) {
						$error = $match->modify();
					} else {
						$error = db_data::ERROR_NONE;
					}
					if( $error !== db_data::ERROR_NONE ) {
						return array( $match, $error );
					}
				}
			}
		}
		return $nRandom;
	}

	/**
	 * Loads the tree.
	 * $pTree can be db_trees or an integer.
	 *
	 * @param mixed $pTree
	 * @param bool $bBuild
	 */
	public function Load( $pTree, $bBuild = true ) {
		$this->pTree = new TTree( $this, $this );
		$this->pTree->Load( $pTree );
		if( $bBuild ) {
			$this->pTree->Build( $this, $this, $this );
		}
	}

	/**
	 * Throw an exception if the tree is not loaded.
	 */
	public function Loaded() {
		if( $this->pTree === NULL ) {
			throw new Exception('Tree is not loaded.');
		}
	}

	/**
	 * Inserts a team into the tournament.
	 * $pTeam can be a db_teams or an integer.
	 * Returns the code for inserting/deleting a team in a tournament
	 * or FALSE on failure.
	 *
	 * @param mixed $pTeam
	 * @param int $nAction
	 * @return int
	 * /
	public function ActionTournamentTeams( $pTeam, $nAction = db_data::OPERATION_INSERT ) {
		if( !( $pTeam instanceof db_teams ) ) {
			$pTemp = new db_teams();
			$pTemp->fill( array( 'team_id' => $pTeam ) );
			$pTeam = $pTemp;
		}

		$fSuccess = false;
		$nAction = (int)$nAction;
		if( $nAction === db_data::OPERATION_INSERT ) {
			$tournament_team = new db_tournament_teams();
			$tournament_team->tournament_id = $this->pTournament->Get()->tournament_id;
			$tournament_team->team_id = $pTeam->team_id;
			$fSuccess = $tournament_team->insert();
		} elseif( $nAction === db_data::OPERATION_DELETE ) {
			$tournament_team = new db_tournament_teams();
			$tournament_team->fill( array( 'AND' => array( 'tournament_id' => $this->pTournament->Get()->tournament_id, 'team_id' => $pTeam->team_id ) ) );
			$fSuccess = $tournament_team->delete();
		}
		return $fSuccess;
	}

	/**
	 * Inserts a match into the tournament.
	 * $pMatch must be a db_matches.
	 * Returns the code for inserting/deleting a team in a tournament
	 * or FALSE on failure.
	 *
	 * @param db_matches $pMatch
	 * @param int $nAction
	 * @return bool
	 * /
	public function ActionMatches( db_matches $pMatch, $nAction = db_data::OPERATION_INSERT ) {
		$fSuccess = false;
		$nAction = (int)$nAction;
		if( $nAction === db_data::OPERATION_INSERT ) {
			// Create links in DB
			$pMatch->tournament_id = $this->pTournament->Get()->tournament_id;
			$pMatch->end_time = TMatch::TIMESTAMP_MATCHPROGRESS;
			$fSuccess = $pMatch->insert();

			if( $fSuccess === db_data::ERROR_NONE ) {
				// Create links in code
				$oMatch = new TMatch( $this );
				$oMatch->Load( $pMatch );
				$oMatch->Build();
				$this->AttachMatch( $oMatch );
			}
		}

		return $fSuccess;
	}

	/**
	 * Inserts a matchTeam into the match.
	 * $pMatch must be a TMatch.
	 * $pTeam can be a db_teams or an integer.
	 * Returns the code for inserting/deleting a team in a match
	 * or FALSE on failure.
	 *
	 * @param TMatch $pMatch
	 * @param int $nAction
	 * @return int
	 * /
	public function ActionMatchTeams( TMatch $pMatch, $pTeam, $nAction = db_data::OPERATION_INSERT ) {
		// The match MUST be in the tournament
		if( $this->pTournament->SearchMatch( $pMatch ) === NULL ) {
			return false;
		}

		if( !( $pTeam instanceof db_teams ) ) {
			$pTemp = new db_teams();
			$pTemp->fill( array( 'team_id' => $pTeam ) );
			$pTeam = $pTemp;
		}

		$fSuccess = false;
		$nAction = (int)$nAction;
		if( $nAction === db_data::OPERATION_INSERT ) {
			$match_team = new db_match_teams();
			$match_team->match_id = $pMatch->Get()->match_id;
			$match_team->team_id = $pTeam->team_id;
			$fSuccess = $match_team->insert();
		} elseif( $nAction === db_data::OPERATION_DELETE ) {
			$match_team = new db_match_teams();
			$match_team->fill( array( 'AND' => array( 'match_id' => $pMatch->Get()->match_id, 'team_id' => $pTeam->team_id ) ) );
			$fSuccess = $match_team->delete();
		}

		if( $fSuccess === db_data::ERROR_NONE ) {
			$pMatch->Build( $this );
		}

		return $fSuccess;
	}

	/**
	 * Enter a score for a match.
	 * $pMatch can be a TMatch or an integer.
	 *
	 * @param mixed $nMatchId
	 * @param int $nTeamId
	 * @param int $nScore
	 * @return bool
	 * /
	public function EnterScore( $pMatch, $nTeamId, $nScore ) {
		$this->Built();

		return $this->pTournament->EnterScore( $pMatch, $nTeamId, $nScore );
	}

	/**
	 * This function will check the last one has already played alone (autowin)
	 * If it is the case, the first one found in the list will be swapped with the last one.
	 * Returns true when no problem occurs... (even or teams swapped).
	 * Returns false when no swapped occured.
	 * TODO For now, the one that has no autowin is taken... It could be better to take
	 *      the one that has the less autowin
	 *
	 * @param db_tournament_teams[] $tournament_teams
	 * @return bool
	 */
	protected function CheckAutowin( &$tournament_teams ) {
		// First, is it odd ?
		$c = count( $tournament_teams );
		if( $c % 2 === 0 ) { // Even... no problem
			return true;
		}

		// The last team has already do an autowin ?
		if( $this->CheckAutowinTeam( $tournament_teams[ $c - 1 ]->team_id ) === false ) {
			return true;
		}

		// We need to swap
		for( $i = 0; $i < $c - 1; $i++ ) {
			if( $this->CheckAutowinTeam( $tournament_teams[ $i ]->team_id ) === false ) {
				// We found one ! Let's go swapping
				$temp = $tournament_teams[ $c - 1 ];
				$tournament_teams[ $c - 1 ] = $tournament_teams[ $i ];
				$tournament_teams[ $i ] = $temp;
				return true;
			}
		}

		return false;
	}

	/**
	 * Returns true if the team already had an autowin.
	 *
	 * @param int $nTeamId
	 * @return bool
	 */
	protected function CheckAutowinTeam( $nTeamId ) {
		global $db, $sys_conf;

		// Special query
		$req = $db->sql_query( 'SELECT b.match_id FROM ' . $sys_conf['db']['table_prefix'] . 'matches AS a, ' . $sys_conf['db']['table_prefix'] . 'match_teams AS b WHERE a.match_id=b.match_id AND a.tournament_id=\'' . $this->pTree->GetTournament()->tournament_id . '\' AND b.team_id=\'' . $nTeamId . '\' ORDER BY b.match_id ASC' );
		$line = $db->sql_numrows( $req );
		$data = $db->sql_fetchrowset( $req );
		for( $i = 0; $i < $line; $i++ ) {
			$req_opponent = $db->sql_query( 'SELECT team_id FROM ' . $sys_conf['db']['table_prefix'] . 'match_teams WHERE match_id=\'' . $data[$i]['match_id'] . '\' AND team_id!=\'' . $nTeamId . '\'' );
			$line_opponent = $db->sql_numrows( $req_opponent );
			if( $line_opponent === 0 ) {
				return true;
			}
		}

		return false;
	}

	public function draw() { }

	/**
	 * Callback
	 */
	public function OnScoreModified( TMatch $pCaller, TMatchTeam $pMatchTeam ) {

	}

	/**
	 * Callback
	 */
	public function OnEndMatch( TMatch $pCaller ) {

	}

	public function _Debug( $code ) {
		$strFirst = str_repeat( '>', $code );

		echo $strFirst . 'Rule<br />';
		$this->pTree->_Debug( $code + 1 );
	}

	public function getTree() {
		$this->Loaded();

		return $this->pTree;
	}

	public function setFake( $bFake ) {
		$this->bFake = (bool)$bFake;
	}

	public function setRandom( $nRandom ) {
		$this->nRandom = (int)$nRandom;
	}
};
?>