<?php
/***********************************************************
 *			RulePool.php
 *		    --------------------
 * Created		: Sat, Mar 11, 2006
 * Copyright		: (C) LookStrike Team
 * WebSite		: http://www.lookstrike.com
 *
 * $Id: RulePool.php,v 1.6 2008/02/02 01:25:30 jsgoupil Exp $
 *
 ***********************************************************/
include_once($sys_conf['path']['real'] . '/modules/class/db/db_trees.php');
include_once($sys_conf['path']['real'] . '/modules/class/tournament/Match.php');
include_once($sys_conf['path']['real'] . '/modules/class/tournament/Rule.php');
include_once($sys_conf['path']['real'] . '/modules/class/draw/DrawPool.php');

class TRulePool extends TRule {
	/**
	 * Constructor
	 */
	public function __construct() {
		parent::__construct();
	}

	/**
	 * Creates the pool.
	 * Returns $error
	 *
	 * @param int $nTournament
	 * @param int $nPrevious
	 * @return int
	 */
	protected function _CreateRule( $nTournament, $nPrevious ) {
		$this->pTree = new TTree( $this, $this );
		$pTempTree = new db_trees();
		$pTempTree->tournament_id = $nTournament;
		$pTempTree->previous = $nPrevious;
		$pTempTree->tree_type = db_trees::TYPE_POOL;
		$this->pTree->Load( $pTempTree );
		if( !$this->bFake ) {
			$error = $pTempTree->insert();
		} else {
			$error = db_data::ERROR_NONE;
		}

		return $error;
	}

	/**
	 * Creates the match.
	 * Returns [0]->db_matches, [1]->$error
	 *
	 * @param mixed $tournament_teams
	 * @param int $i Current Team
	 * @return mixed
	 */
	protected function _CreateMatch( $tournament_teams, $i ) {
		$match = new db_matches();
		$match->tournament_id = $this->pTree->GetTournament()->tournament_id;
		$match->tree_id = $this->pTree->Get()->tree_id;
		if( !$this->bFake ) {
			$error = $match->insert();
		} else {
			$error = db_data::ERROR_NONE;
		}

		return array( $match, $error );
	}

	/**
	 * Saves the match
	 *
	 * @param int $nMatchId
	 * @param int[] $arrTeams
	 * @param int[] $arrScore
	 * @return bool
	 */
	public function SaveMatch( $nMatchId, $arrTeams, $arrScores ) {
		$success = parent::SaveMatch( $nMatchId, $arrTeams, $arrScores );

		if( $success ) {
			$match = new TMatch( $this );
			$match->Load( intval( $nMatchId ) );
			$match->Build( $this );

			// We reload just the match
			$success = $this->getTree()->ReplaceMatch( $match );
		}

		return $success;
	}

	/**
	 * Creates the Pool and its matches.
	 * $nPrevious is the previous tree_id
	 * Returns an array with [0]->db_data, [1]->error_number on error
	 * Or returns true on no error.
	 *
	 * @param int $nTournament
	 * @param int $nPrevious
	 * @return mixed
	 */
	public function Create( $nTournament, $nPrevious ) {
		return TRule::Create( $nTournament, $nPrevious );
	}

	/**
	 * Draws the pool in an image.
	 *
	 * @param string $pathToRoot
	 * @return bool
	 */
	public function draw( $pathToRoot = '.' ) {
		$drawPool = new TDrawPool();
		$drawPool->Load( $this );
		$drawPool->setBackground( $pathToRoot . '/images/trback.png' ); #TODO
		$drawPool->setWinner( new Winner_1() );
		$im = $drawPool->draw();

		if($im !== false) {
			imagepng( $im, $pathToRoot . '/images/generated/' . $this->pTree->Get()->tree_id . '.png' );
			return true;
		} else {
			return false;
		}
	}

	public function GetManualPosition() {
		$arr = array();
		$temp = new db_tournament_teams;
		$c = count( db_data::searchForClass( 'db_tournament_teams', array( 'tournament_id' => $this->pTree->Get()->tournament_id ), '', '', $temp->getPrimary() ) );
		for( $i = 0; $i < $c; $i++ ) {
			$arr[] = $i;
		}
		return $arr;
	}

	public function _Debug( $code ) {
		$strFirst = str_repeat( '>', $code );

		echo $strFirst . 'RulePool<br />';
		$this->pTree->_Debug( $code + 1 );
	}
};
?>