<?php
/***********************************************************
 *			RuleTree.php
 *		    --------------------
 * Created		: Sat, Mar 11, 2006
 * Copyright		: (C) LookStrike Team
 * WebSite		: http://www.lookstrike.com
 *
 * $Id: RuleTree.php,v 1.6 2008/02/02 01:25:30 jsgoupil Exp $
 *
 ***********************************************************/
include_once($sys_conf['path']['real'] . '/modules/class/db/db_trees.php');
include_once($sys_conf['path']['real'] . '/modules/class/tournament/Rule.php');
include_once($sys_conf['path']['real'] . '/modules/class/draw/TreePosition.php');
include_once($sys_conf['path']['real'] . '/modules/class/draw/DrawTree.php');

class TRuleTree extends TRule {
	private $nDepth;
	private $posTable;

	/**
	 * Constructor
	 */
	public function __construct() {
		parent::__construct();
	}

	/**
	 * Creates the tree.
	 * Returns $error
	 *
	 * @param int $nTournament
	 * @param int $nPrevious
	 * @return int
	 */
	protected function _CreateRule( $nTournament, $nPrevious ) {
		$this->pTree = new TTree( $this, $this );
		$pTempTree = new db_trees();
		$pTempTree->tournament_id = $nTournament;
		$pTempTree->previous = $nPrevious;
		$this->pTree->Load( $pTempTree );

		if( $this->GetDepth() === 0 ) {
			$this->nDepth = $this->CalculateDepth( 0 );
		}
		$this->posTable = $this->FindChild();

		$pTempTree->tree_type = $this->nDepth;

		if( !$this->bFake ) {
			$error = $pTempTree->insert();
		} else {
			$error = db_data::ERROR_NONE;
		}

		return $error;
	}

	/**
	 * Creates the match.
	 * Returns [0]->db_matches, [1]->$error
	 *
	 * @param mixed $tournament_teams
	 * @param int $i Current Team
	 * @return mixed
	 */
	protected function _CreateMatch( $tournament_teams, $i ) {
		$c2 = count( $this->posTable );
		$add = intval( $i / $c2 );
		$pos = $i % $c2 + $add;		// We take 1 position on 2

		$match = new db_matches();
		$match->tournament_id = $this->pTree->GetTournament()->tournament_id;
		$match->tree_id = $this->pTree->Get()->tree_id;
		if(isset($this->posTable[ $pos ])) {
			$match->node = $this->posTable[ $pos ];
			if( !$this->bFake ) {
				$error = $match->insert();
			} else {
				$error = db_data::ERROR_NONE;
			}
			return array( $match, $error );
		}
		return array( $match, db_data::ERROR_NOTALLOWED );
	}

	private function isMatchAvailableAtThisNode( $node ) {
		$match = new db_matches();
		$match->fill(array('AND'=>array('tree_id'=>$this->pTree->Get()->tree_id, 'node'=>$node)));

		return ( intval( $match->match_id ) !== 0 );
	}

	private function findSomeone( TTreeNode $currentNode, &$currentNodeInteger ) {
		if( $currentNode->left === NULL || $currentNode->right === NULL ) {	// We are at the end of the tree
			return false;
		}

		// Check node on the left (currentNodeInteger + 1)
		if( $this->isMatchAvailableAtThisNode( $currentNodeInteger + 1 ) ) {
			return true;
		}

		if( $currentNode->left !== NULL ) {
			$left_value = $this->findSomeone( $currentNode->left, $currentNodeInteger + 1 );
			if( $left_value !== false ) {
				return $left_value;
			}
		}

		// Check node on the right (currentNodeInteger + 1)
		if( $this->isMatchAvailableAtThisNode( $currentNodeInteger + 1 ) ) {
			return true;
		}

		if($this->right !== NULL) {
			$right_value = $this->findSomeone( $currentNode->right, $currentNodeInteger + 1 );
			if( $right_value !== false ) {
				return $right_value;
			}
		}
		return false;
	}

	private function findWinnerOfMatch( $nMatchId ) {
		$db_match_teams = new db_match_teams();
		$match_teams = db_data::searchForClass( 'db_match_teams', array('match_id'=>$nMatchId), 'score DESC', '', $db_match_teams->getPrimary() );
		if( count( $match_teams ) > 0 ) {
			return $match_teams[ 0 ]->team_id;
		} else {
			throw new Exception('No Winning Team Found');
			return 0;
		}
	}

	private function checkForNewMatches( $nMatchId ) {
		// What is the node Number ?
		$match = new db_matches();
		$match->fill(array('match_id'=>$nMatchId));
		$nodeWinningInteger = (int)$match->node;

		// We create the tree
		$tree = new TTreePosition( $this->nDepth );
		$firstNode = $tree->getFirstNode();
		$number = 0;
		$nodeWinning = $firstNode->findNode( $nodeWinningInteger, $number );
		$nodeVS = $firstNode->findVS( $nodeWinning );

		// If the $nodeVS is null, we are at the top of the tree
		if( $nodeVS !== NULL ) {
			// Do we have a match in the VS Node ?
			$number = 0;
			$nodeVSInteger = $firstNode->findPrefix( $nodeVS, $number );
			$match = new db_matches();
			$match->fill(array('AND'=>array('tree_id'=>$this->pTree->Get()->tree_id, 'node'=>$nodeVSInteger)));

			// We didn't find any match at this position
			if( intval( $match->match_id ) === 0 ){
				// Search if we can find another children of the $nodeVS
				if ($this->findSomeone( $nodeVS, $nodeVSInteger ) === false) { // Nobody Found
					// We have an autowin!
					$number = 0;
					$nodeNext = $firstNode->findFather( $nodeWinning );
					$nodeNextInteger = $firstNode->findPrefix( $nodeNext, $number );

					$newMatch = new db_matches();
					$newMatch->tournament_id = $this->pTree->GetTournament()->tournament_id;
					$newMatch->tree_id = $this->pTree->Get()->tree_id;
					$newMatch->node = $nodeNextInteger;
					$newMatch->start_time = db_data::GetNow(db_data::DB_TIME_TIMESTAMP);
					$newMatch->end_time = db_data::GetNow(db_data::DB_TIME_TIMESTAMP); // TODO Do we need this +1 in second ?
					$newMatch->insert();

					$newTeam1 = new db_match_teams();
					$newTeam1->match_id = $newMatch->match_id;
					$newTeam1->team_id = $this->findWinnerOfMatch( $nMatchId ); // The one who won the $nMatchId
					$newTeam1->score = 1;
					$newTeam1->insert();

					$match = new TMatch( $this );
					$match->Load( intval( $newMatch->match_id ) );
					$match->Build( $this );

					// We reload just the match
					$this->getTree()->AttachMatch( $match );

					// Check for new matches available since we added a finished one
					$this->checkForNewMatches( $newMatch->match_id );
				} else { // Someone Found
					// We wait for these people to finish
				}
			} else { // We found a match at this position
				// Is the match over ?
				if( $match->end_time !== '0000-00-00 00:00:00') {
					$number = 0;
					$nodeNext = $firstNode->findFather( $nodeWinning );
					$nodeNextInteger = $firstNode->findPrefix( $nodeNext, $number );

					$newMatch = new db_matches();
					$newMatch->tournament_id = $this->pTree->GetTournament()->tournament_id;
					$newMatch->tree_id = $this->pTree->Get()->tree_id;
					$newMatch->node = $nodeNextInteger;
					$newMatch->insert();

					$newTeam1 = new db_match_teams();
					$newTeam1->match_id = $newMatch->match_id;
					$newTeam1->team_id = $this->findWinnerOfMatch( $nMatchId ); // The one who won the $nMatchId
					$newTeam1->insert();

					$newTeam2 = new db_match_teams();
					$newTeam2->match_id = $newMatch->match_id;
					$newTeam2->team_id = $this->findWinnerOfMatch( $match->match_id ); // The one who won the $match->match_id
					$newTeam2->insert();

					$match = new TMatch( $this );
					$match->Load( intval( $newMatch->match_id ) );
					$match->Build( $this );

					// We reload just the match
					$this->getTree()->AttachMatch( $match );
				} else {
					// Nobody is winning yet
				}
			}
		}
	}

	/**
	 * Saves the match
	 *
	 * @param int $nMatchId
	 * @param int[] $arrTeams
	 * @param int[] $arrScore
	 * @return bool
	 */
	public function SaveMatch( $nMatchId, $arrTeams, $arrScores ) {
		$success = parent::SaveMatch( $nMatchId, $arrTeams, $arrScores );

		if( $success ) {
			$match = new TMatch( $this );
			$match->Load( intval( $nMatchId ) );
			$match->Build( $this );

			// We reload just the match
			$success = $this->getTree()->ReplaceMatch( $match );

			$this->checkForNewMatches( $nMatchId );
		}

		return $success;
	}

	/**
	 * Creates the Tree and its matches
	 * $nPrevious is the previous tree_id
	 * $nDepth is the depth of the tree starting with 1.
	 * Returns an array with [0]->db_data, [1]->error_number on error
	 * Or returns true on no error.
	 *
	 * @param int $nTournament
	 * @param int $nPrevious
	 * @param int $nDepth
	 * @return mixed
	 */
	public function Create( $nTournament, $nPrevious, $nDepth = 0 ) {
		$this->nDepth = (int)$nDepth;

		return TRule::Create( $nTournament, $nPrevious );
	}

	/**
	 * Calculates the depth required for the tournament loaded or the number
	 * of teams contained in $nNumberTeams.
	 *
	 * @param int $nNumberTeams
	 * @return int
	 */
	public function CalculateDepth( $nNumberTeams = 0 ) {
		$nNumberTeams = (int)$nNumberTeams;
		if( $nNumberTeams === 0 ) {
			// Fetch number of teams in tournament
			$temp = new db_tournament_teams;
			$nNumberTeams = count( db_data::searchForClass( 'db_tournament_teams', array( 'tournament_id' => $this->pTree->Get()->tournament_id ), '', '', $temp->getPrimary() ) );
		}

		$nDeep = $nNumberTeams * 2;
		for ($i = 0; $i < $nDeep; $i++) {
			if ($nDeep / (2 << $i) <= 1) {
				break;
			}
		}

		return $i;
	}

	/**
	 * Loads the tree.
	 * $pTree can be db_trees or an integer.
	 *
	 * @param mixed $pTree
	 * @param bool $bBuild
	 */
	public function Load( $pTree, $bBuild = true ) {
		parent::Load( $pTree, $bBuild );
		
		$this->nDepth = $this->pTree->Get()->tree_type;
	}

	/**
	 * Returns Child of the tree.
	 * Index is calculated by postfix beginning at 0.
	 *
	 * @return int[]
	 */
	private function FindChild() {
		$this->temp_val = 0;
		$position_table = array();
		$this->_FindChild( $position_table, 0 );
		return $position_table;
	}

	private function _FindChild( &$position_table, $position ) {
		$this->temp_val++;

		if ( $this->temp_val === $this->GetDepth() ) {
			$position_table[] = $position;
			$this->temp_val--;
			return $position;
		}
		$position++;
		$position = $this->_FindChild( $position_table, $position ); // Left
		$position++;
		$position = $this->_FindChild( $position_table, $position ); // Right
		$this->temp_val--;
		return $position;
	}

	public function GetDepth() {
		if( $this->nDepth === NULL ) {
			return (int)$this->pTree->Get()->tree_type;
		} else {
			return $this->nDepth;
		}
	}

	public function setDepth($depth) {
		$this->nDepth = $depth;
	}

	/**
	 * Draws the pool in an image.
	 *
	 * @param string $pathToRoot
	 * @return bool
	 */
	public function draw( $pathToRoot = '.' ) {
		$drawTree = new TDrawTree();
		$drawTree->load( $this );
		$drawTree->setBackground( $pathToRoot . '/images/trback.png' ); #TODO
		$drawTree->setWinner( new Winner_1() );
		$im = $drawTree->draw();

		if($im !== false) {
			imagepng( $im, $pathToRoot . '/images/generated/' . $this->pTree->Get()->tree_id . '.png' );
			return true;
		} else {
			return false;
		}
	}

	public function GetManualPosition() {
		if( $this->GetDepth() === 0 ) {
			$this->nDepth = $this->CalculateDepth( 0 );
		}

		$arr = array();
		$c = pow( 2, $this->GetDepth() );
		for( $i = 0; $i < $c; $i++ ) {
			$arr[] = $i;
		}
		return $arr;
	}

	public function _Debug( $code ) {
		$strFirst = str_repeat( '>', $code );

		echo $strFirst . 'RuleTree<br />';
		$this->pTree->_Debug( $code + 1 );
	}
};
?>