<?php
/***********************************************************
 *			Tournament.php
 *		    ----------------------
 * Created		: Thu, Mar 9, 2006
 * Copyright		: (C) LookStrike Team
 * WebSite		: http://www.lookstrike.com
 *
 * $Id: Tournament.php,v 1.3 2008/02/02 01:25:30 jsgoupil Exp $
 *
 ***********************************************************/
include_once($sys_conf['path']['real'] . '/modules/class/tournament/Match.php');
include_once($sys_conf['path']['real'] . '/modules/class/tournament/TournamentTeam.php');
include_once($sys_conf['path']['real'] . '/modules/class/db/db_matches.php');
include_once($sys_conf['path']['real'] . '/modules/class/db/db_tournaments.php');

interface ITournamentListener {
}

class TTournament {
	protected $pTournamentListener;
	protected $pTournament;
	protected $pMatches = array();
	protected $pTournamentTeams = array();

	public function __construct( $pTournamentListener = NULL ) {
		$this->pTournamentListener = $pTournamentListener;
	}

	/**
	 * Loads a tournament.
	 * $pTournament can be a db_tournaments or an integer.
	 * Returns true on success
	 *
	 * @param mixed $pTournament
	 * @return bool
	 */
	public function Load( $pTournament ) {
		if( $pTournament instanceof db_tournaments ) {
			$this->pTournament = $pTournament;
		} else {
			$this->pTournament = new db_tournaments();
			$this->pTournament->fill( array( $this->pTournament->getPrimary() => $pTournament ) );
		}

		if( !$this->pTournament->IsLoaded() ) {
			return false;
		}

		return true;
	}

	/**
	 * Throw an exception is the tournament is not loaded
	 */
	public function Loaded() {
		if( $this->pTournament === NULL || !$this->pTournament->IsLoaded() ) {
			throw new Exception('Tournament is not loaded.');
		}
	}

	/**
	 * Builds Matches into the class.
	 *
	 * @param IMatchtListener $pMatchListener
	 * @param ITournamentTeamListener $pTournamentTeamListener
	 * @param IMatchTeamListener $pMatchTeamListener
	 */
	public function Build( $pMatchListener = NULL, $pTournamentTeamListener = NULL, $pMatchTeamListener = NULL ) {
		$this->Loaded();

		$match = new db_matches();
		$matches = db_data::searchForClass( 'db_matches', array( 'tournament_id' => $this->pTournament->tournament_id ), '', '', $match->getPrimary() );

		$c = count( $matches );
		for( $i = 0; $i < $c; $i++ ) {
			$oMatch = new TMatch( $pMatchListener );
			$oMatch->Load( $matches[$i] );
			$oMatch->Build();
			$this->AttachMatch( $oMatch );
		}

		$tournament_team = new db_tournament_teams();
		$tournament_teams = db_data::searchForClass( 'db_tournament_teams', array( 'tournament_id' => $this->pTournament->tournament_id ), '', '', $tournament_team->getPrimary() );

		$c = count( $tournament_teams );
		for( $i = 0; $i < $c; $i++ ) {
			$oTournamentTeams = new TTournamentTeam( $pTournamentTeamListener );
			$oTournamentTeams->Load( $tournament_teams[$i] );
			$oTournamentTeams->Build();
			$this->AttachTournamentTeam( $oTournamentTeams );
		}
	}

	/**
	 * Returns the tournament.
	 *
	 * @return db_tournaments
	 */
	public function Get() {
		return $this->pTournament;
	}

	/**
	 * Attaches a match to the tournament
	 */
	public function AttachMatch( TMatch $pMatch ) {
		$this->pMatches[] = $pMatch;
	}

	/**
	 * Attaches a team to the tournament
	 */
	public function AttachTournamentTeam( TTournamentTeam $pTournamentTeam ) {
		$this->pTournamentTeams[] = $pTournamentTeam;
	}


	/**
	 * Replaces the pMatch in the tournament
	 *
	 * @param TMatch $pMatch
	 * @return bool
	 */
	public function ReplaceMatch( TMatch $pMatch ) {
		$c = count( $this->pMatches );
		if( $pMatch instanceof TMatch ) {
			for( $i = 0; $i < $c; $i++ ) {
				if( intval( $this->pMatches[$i]->Get()->match_id ) === intval( $pMatch->Get()->match_id ) ) {
					$this->pMatches[$i] = $pMatch;
					return true;
				}
			}
		}

		return false;
	}

	/**
	 * Enter a score for a match.
	 * $pMatch can be a TMatch or an integer.
	 *
	 * @param mixed $nMatchId
	 * @param int $nTeamId
	 * @param int $nScore
	 * @return bool
	 */
	public function EnterScore( $pMatch, $nTeamId, $nScore ) {
		$nTeamId = (int)$nTeamId;
		$nScore = (int)$nScore;

		if( ( $pMatch = $this->SearchMatch( $pMatch ) ) !== NULL ) {
			return $pMatch->EnterScore( $nTeamId, $nScore );
		}

		return false;
	}

	/**
	 * Searches for a match into the tournament
	 * $pMatch can be a TMatch or an integer.
	 * Returns the matches if found, otherwise returns null.
	 *
	 * @param mixed $pMatch
	 * @return TMatch
	 */
	public function SearchMatch( $pMatch ) {
		$c = count( $this->pMatches );
		if( $pMatch instanceof TMatch ) {
			for( $i = 0; $i < $c; $i++ ) {
				if( $this->pMatches[$i] === $pMatch ) {
					return $this->pMatches[$i];
				}
			}
		} else {
			$pMatch = (int)$pMatch;
			for( $i = 0; $i < $c; $i++ ) {
				if( intval( $this->pMatches[$i]->Get()->match_id ) === $pMatch ) {
					return $this->pMatches[$i];
				}
			}
		}

		return NULL;
	}

	public function _Debug( $code ) {
		$strFirst = str_repeat( '>', $code );

		echo $strFirst . 'Tournament.id = ' . $this->pTournament->tournament_id . '<br />';

		$c = count( $this->pTournamentTeams );
		for( $i = 0; $i < $c; $i++ ) {
			$this->pTournamentTeams[$i]->_Debug( $code + 1 );
		}

		$c = count( $this->pMatches );
		for( $i = 0; $i < $c; $i++ ) {
			$this->pMatches[$i]->_Debug( $code + 1 );
		}
	}

	public function getTournament() {
		$this->Loaded();

		return $this->pTournament;
	}

	public function getMatches() {
		return $this->pMatches;
	}

	public function getTournamentTeams() {
		return $this->pTournamentTeams;
	}
};
?>