<?php
/***********************************************************
 *			Tree.php
 *		    ----------------
 * Created		: Sun, Apr 23, 2006
 * Copyright		: (C) LookStrike Team
 * WebSite		: http://www.lookstrike.com
 *
 * $Id: Tree.php,v 1.1 2007/02/15 15:04:42 jsgoupil Exp $
 *
 ***********************************************************/
include_once($sys_conf['path']['real'] . '/modules/class/tournament/Tournament.php');
include_once($sys_conf['path']['real'] . '/modules/class/db/db_trees.php');

interface ITreeListener {
}

class TTree extends TTournament {
	protected $pTreeListener;
	protected $pTree;

	public function __construct( $pTreeListener = NULL, $pTournamentListener = NULL ) {
		parent::__construct( $pTournamentListener );
		$this->pTreeListener = $pTreeListener;
	}

	/**
	 * Loads a tree.
	 * $pTree can be a db_trees or an integer.
	 * Returns true on success
	 *
	 * @param mixed $pTree
	 * @return bool
	 */
	public function Load( $pTree ) {
		if( $pTree instanceof db_trees ) {
			$this->pTree = $pTree;
		} else {
			$this->pTree = new db_trees();
			$this->pTree->fill( array( $this->pTree->getPrimary() => $pTree ) );
		}
		// We don't check if pTree IsLoaded because if we create a new one,
		// The tree won't have any id.
		return parent::Load( $this->pTree->tournament_id );
	}

	/**
	 * Throw an exception is the tree and tournament are not loaded
	 */
	public function Loaded() {
		if( $this->pTree === NULL || !$this->pTree->IsLoaded() ) {
			throw new Exception('Tree is not loaded.');
		}
		parent::Loaded();
	}

	/**
	 * Builds Matches into the class.
	 *
	 * @param IMatchtListener $pMatchListener
	 * @param ITournamentTeamListener $pTournamentTeamListener
	 * @param IMatchTeamListener $pMatchTeamListener
	 */
	public function Build( $pMatchListener = NULL, $pTournamentTeamListener = NULL, $pMatchTeamListener = NULL ) {
		$this->Loaded();

		$match = new db_matches();
		$matches = db_data::searchForClass( 'db_matches', array( 'tree_id' => $this->pTree->tree_id ), '', '', $match->getPrimary() );

		$c = count( $matches );
		for( $i = 0; $i < $c; $i++ ) {
			$oMatch = new TMatch( $pMatchListener );
			$oMatch->Load( $matches[$i] );
			$oMatch->Build();
			$this->AttachMatch( $oMatch );
		}

		$tournament_team = new db_tournament_teams();
		$tournament_teams = db_data::searchForClass( 'db_tournament_teams', array( 'tournament_id' => $this->pTournament->tournament_id ), '', '', $tournament_team->getPrimary() );

		$c = count( $tournament_teams );
		for( $i = 0; $i < $c; $i++ ) {
			$oTournamentTeams = new TTournamentTeam( $pTournamentTeamListener );
			$oTournamentTeams->Load( $tournament_teams[$i] );
			$oTournamentTeams->Build();
			$this->AttachTournamentTeam( $oTournamentTeams );
		}
	}

	/**
	 * Returns the tree.
	 *
	 * @return db_trees
	 */
	public function Get() {
		return $this->pTree;
	}

	/**
	 * Returns the tournament
	 *
	 * @return db_tournaments
	 */
	public function GetTournament() {
		return parent::Get();
	}

	public function _Debug( $code ) {
		$strFirst = str_repeat( '>', $code );

		echo $strFirst . 'Tree.id = ' . $this->pTree->tree_id . '<br />';

		parent::_Debug( $code );
	}
};
?>