<?php
/***********************************************************
 *			function.php
 *		    --------------------
 * Created		: Wed, Nov 19, 2003
 * Copyright		: (C) LookStrike Team
 * WebSite		: http://www.lookstrike.com
 *
 * $Id: function.php,v 1.12 2008/01/28 03:16:14 jsgoupil Exp $
 *
 ***********************************************************/
if(!defined('IN_LS'))die('You are not allowed to access to this page.');

/**
 * Transforme l'adresse IP sous format dcimal avec point
 * en format hexadcimal pour stoquage.
 *
 * @param string $dotquad_ip Adresse IP (ex.: 192.168.0.1)
 * @return string Adresse IP sous format hexadcimal
 */
function encode_ip($dotquad_ip) {
	$ip_sep = explode('.', $dotquad_ip);
	return sprintf('%02x%02x%02x%02x', $ip_sep[0], $ip_sep[1], $ip_sep[2], $ip_sep[3]);
}

/**
 * Transforme l'adresse IP sous format hxadcimal.
 * en format dcimal avec point pour affichage.
 * ventuellement, un port peut tre plac aprs l'IP (:port)
 *
 * @param string $hex Adresse IP (ex.: C0A80001)
 * @return string Adresse IP sous format dcimal avec point
 */
function decode_ip($hex) {
	$toreturn = sprintf('%d.%d.%d.%d', intval(substr($hex, 0, 2), 16), intval(substr($hex, 2, 2), 16), intval(substr($hex, 4, 2), 16), intval(substr($hex, 6, 2), 16));
	$toreturn .= substr($hex, 8);
	return $toreturn;
}

/**
 * Retourne la class suivante pour une ligne d'un tableau.
 *
 * @param int $restart Si vaut 1, alors on redmarre avec la couleur 1
 * @return string
 */
function next_color($restart = 0) {
	global $sys_conf;
	static $color=0;
	if($restart === 1){ $color = NULL; }
	if($color === 1){ $couleur = 'row2'; $color = 2; }
	else { $couleur = 'row1'; $color = 1; }
	return $couleur;
}

/**
 * Retourne le tableau de "Updated" avec le texte et le mot.
 * Redirection (ainsi qu'un META)
 *
 * @param string $text Texte  afficher
 * @param string $url Adresse URL de retour
 * @param int $meta_time Temps d'attente avant la redirection (NULL si aucune redirection auto)
 * @return LSTable Table  afficher
 */
function display_updated($text, $url, $meta_time = 2) {
	global $language;
	$null = NULL;
	$meta = '';
	if($meta_time !== NULL && $url !== NULL) {
		$meta = '<meta http-equiv="refresh" content="' . $meta_time . '; url=' . $url . '">';
	}
	$number_row = ($url !== NULL) ? 2 : 1;
	$table = new LSTable($number_row, 1, '60%', $null);
	$table->setTitle($language['common_updated']);
	$table->addCellAttribute(0, 0, 'height', '50');
	$table->setText(0, 0, $text.$meta);
	if($url !== NULL) {
		$table->addCellAttribute(1, 0, 'align', 'right');
		$table->setText(1, 0, '<font class="littletext">[ <a href="' . $url . '">' . $language['common_redirection'] . '</a> ]</font>');
	}
	return $table;
}

/**
 * Retourne le tableau de "Erreur" avec le texte et le mot.
 *
 * @param string $text Texte  afficher
 * @param string $url Adresse URL de retour
 * @param string $url2
 * @return LSTable Table  afficher
 */
function display_error($text, $url = 'javascript:history.back(1)', $url2 = NULL) {
	global $language;

	// In that case, $text is the instance, $url is the error code, $url2 is the url
	if ($text instanceof db_data) {
		$null = NULL;
		$number_row = ($url2 !== NULL) ? 2 : 1;
		$table = new LSTable($number_row, 1, '60%', $null);
		$table->setTitle($language['common_error']);
		$display = $text->getErrorLanguage($url);
		$table->addCellAttribute(0, 0, 'height', '50');
		$table->setText(0, 0, $display);
		if($url2 !== NULL) {
			$table->addCellAttribute(1, 0, 'align', 'right');
			$table->setText(1, 0, '<font class="littletext">[ <a href="' . $url2 . '">' . $language['common_back'] . '</a> ]</font>');
		}
		return $table;
	} else {
		$null = NULL;
		$number_row = ($url !== NULL) ? 2 : 1;
		$table = new LSTable($number_row, 1, '60%', $null);
		$table->setTitle($language['common_error']);
		$table->addCellAttribute(0, 0, 'height', '50');
		$table->setText(0, 0, $text);
		if($url !== NULL) {
			$table->addCellAttribute(1, 0, 'align', 'right');
			$table->setText(1, 0, '<font class="littletext">[ <a href="' . $url . '">' . $language['common_back'] . '</a> ]</font>');
		}
		return $table;
	}
}

/**
 * Enleve les Espaces superflus dans un tableau _POST ou _GET...
 * Si $to_html est vrai, alors on enlve les \ en trop et les
 * caractres spciaux
 *
 * @param mixed $table Tableau  modifier
 * @param bool $to_html Si vrai, on ajoute les caractres spciaux
 */
function format_to_screen(&$table,$to_html = false) {
	if(is_array($table)) {
		while(list($key, $value) = each($table)) {
			// Si multidimensionnel
			if(is_array($table[$key])) {
				while(list($key2, $value2) = each($table[$key])) {
					$table[$key][$key2] = trim($value2);
					if($to_html === true) {
						$table[$key][$key2] = stripslashes($table[$key][$key2]);
						$table[$key][$key2] = htmlspecialchars($table[$key][$key2]);
					}
				}
				@reset($table[$key]);
			} else {
				$table[$key] = trim($value);
				if($to_html === true) {
					$table[$key] = stripslashes($value);
					$table[$key] = htmlspecialchars($table[$key]);
				}
			}
		}
	}
	@reset($table);
}

/**
 * Enleve dans le $query, le $remove.
 *
 * @param string $query Valeur du Query
 * @param string $remove Variable  supprimer
 * @return string Nouveau Query obtenu
 */
function remove_query($query, $remove) {
	if(!empty($query)) {		// Bug http://bugs.php.net/bug.php?id=28862, normally it's if(isset($query[0]) && $query == '?')
		if($query[0] === '?') {
			$query = substr($query, 1);
		}
	}
	$query=str_replace('&', '&amp;', eregi_replace('(' . $remove . '=[a-zA-Z0-9\.\-]*\&?)|(\&?' . $remove . '=[a-zA-Z0-9\.\-]*)', '', $query));
	return $query;
}

/**
 * Insre dans $array une ligne $value  la position $pos.
 *
 * @param mixed[] $array Tableau  modifier
 * @param mixed $value Ligne  ajouter
 * @param int $pos Position de la nouvelle ligne
 */
function array_insert(&$array, $value, $pos)
{
	if (!is_array($array)) {
		return false;
	}
	$last = array_splice($array, $pos);
	array_push($array, $value);
	$array = array_merge($array, $last);
}

/**
 * Retourne les infos de $array si elles existent sinon, celle de $data.
 *
 * @param mixed[] $array Tableau de donnes
 * @param ressource $data Donnes de la base de donnes
 * @param string $prefix Valeur de prfix avant chaque prochain argument
 * @param string ... Variable index du tableau $array
 * @return mixed[] Nouveau Tableau obtenu
 */
function format_html(&$array, $data, $prefix) {
	$variable_number = func_num_args();
	$to_return = array();
	for($i = 3; $i < $variable_number; $i++) {
		$temp = func_get_arg($i);
		$to_return[$prefix . $temp] = (isset($array[$prefix . $temp])) ? $array[$prefix . $temp] : (($data === NULL) ? NULL : $data[$temp]);
	}
	return $to_return;
}

/**
 * Vrifie les champs qui doit tre requis. Rempli le tableau.
 * si le champ est vide, null ou n'existe pas.
 *
 * @param mixed $array Tableau de donnes
 * @param string $prefix Valeur de prfix avant chaque prochain argument
 * @param string ... Variable index du tableau $array
 * @return string[] Champs requis
 */
function check_required(&$array, $prefix) {
	$variable_number = func_num_args();
	$field_required = array();
	for($i = 2; $i < $variable_number; $i++) {
		$temp = func_get_arg($i);
		if(!isset($array[$prefix . $temp]) || $array[$prefix . $temp] === '' || $array[$prefix . $temp] === NULL) {
			$field_required[] = $temp;
		}
	}
	return $field_required;
}

/**
 * Format le texte x='".$x."' pour un future UPDATE sql.
 *
 * @param mixed $array Tableau de donnes
 * @param string $prefix Valeur de prfix avant chaque prochain argument
 * @param string ... Variable index du tableau $array
 * @return string Valeur  mettre dans un sql_query
 */
function to_update(&$array, $prefix) {
	$variable_number = func_num_args();
	$to_return = '';
	for($i = 2; $i< $variable_number; $i++) {
		$to_return .= func_get_arg($i) . '=\'' . $array[$prefix.func_get_arg($i)] . '\',';
	}
	$to_return = substr($to_return, 0, strlen($to_return) - 1);
	return $to_return;
}

/**
 * Calcul le nombre de page  afficher si trop de rsultats.
 *
 * @param int $limit Nombre O on est rendu (LIMIT $limit,xxx)
 * @param int $maxpage Nombre maximal de page  afficher
 * @param string $query $_SERVER['QUERY_STRING']
 * @param int $nombre_page Nombre d'Affichage par Page
 * @param int $max_number Nombre de chiffre  afficher avant de mettre des ...
 * @return Valeur  afficher
 */
function page_number($limit, $maxpage, $query, $nombre_page, $max_number) {
	global $language;

	$query = remove_query($query, 'limit');

	$p_page = floor($limit / $nombre_page);
	$p_page++;

	$limit_prec = $limit - $nombre_page;
	$limit_next = $limit + $nombre_page;

	if($limit_prec < 0) $limit_prec = 0;
	if($limit_prec >= 0 && $limit !== 0) $page_prec = '<a href="?limit=' . $limit_prec . '&amp;' . $query . '">' . $language['common_prev'] . '</a>';
	else $page_prec = '';

	if($p_page < $maxpage) $page_next = '<a href="?limit=' . $limit_next . '&amp;' . $query . '">' . $language['common_next'] . '</a>';
	else $page_next = '';

	$page_page = '';
	$specify_min = '';
	$specify_max = '';
	if($maxpage > $max_number) {
		$min_start = $p_page - intval($max_number / 2);
		$last_start= $p_page + intval($max_number / 2);
		if($min_start <= 0) {
			$last_start += (-$min_start + 1);
		}

		if($min_start <= 1) {
			$min_start = 1;
		} else {
			$specify_min = '...';
		}

		if($last_start >= $maxpage) {
			$min_start -= $last_start - $maxpage;
			$last_start = $maxpage;
		}
		else $specify_max = '...';
	} else {
		$min_start = 1;
		$last_start = $maxpage;
	}
	for($i = $min_start; $i <= $last_start; $i++) {
		$thelimit = $i * $nombre_page - $nombre_page;
		if((int)$p_page !== $i) $page_page .= '<a href="?limit=' . $thelimit . '&amp;' . $query . '">' . $i . '</a> ';
		else $page_page .= $i . ' ';
	}
	if($maxpage === 1) {
		return NULL;
	} else {
		return $page_prec . ' ' . $specify_min . ' ' . $page_page . ' ' . $specify_max . ' ' . $page_next;
	}
}

/**
 * En fournissant un timestamp unix yyyymmddhhiiss
 * ou yyyy-mm-dd hh:ii:ss .
 * La fonction retourne en date le $output que l'on veut
 *
 * @param string $time_stamp Valeur TimeStamp
 * @param string $output Valeur de sortie suivant les valeurs de la fonction date
 * @return string Date Formate
 */
function transform_timestamp($time_stamp, $output = 'd-m-Y H:i:s') {
	if(strlen($time_stamp) === 14) {
		return date($output, mktime(substr($time_stamp, 8, 2), substr($time_stamp, 10, 2), substr($time_stamp, 12, 2), substr($time_stamp, 4, 2), substr($time_stamp, 6, 2), substr($time_stamp, 0, 4)));
	} else {
		return date($output, mktime(substr($time_stamp, 11, 2), substr($time_stamp, 14, 2), substr($time_stamp, 17, 2), substr($time_stamp, 5, 2), substr($time_stamp, 8, 2), substr($time_stamp, 0, 4)));
	}
}

/**
 * Permet d'executer la fonction htmlspecialchars sur toutes
 * les donnes d'une variable (qui peut etre tableau).
 *
 * @param string[] $table
 */
function transform_specialchars(&$table) {
	if($table !== NULL) {
		if(is_array($table)) {
			while(list($key, $value) = each($table)) {
				if(is_array($table[$key])) {
					transform_specialchars($table[$key]);
				} else {
					if($value !== NULL) {
						$table[$key] = htmlspecialchars($value);
					}
				}
			}
		} else {
			$table = htmlspecialchars($table);
		}
	}
}

/**
 * Replace les patterns d'une string en insrant des choses
 * devant ce pattern et derrire. Ceci conserve la casse du string.
 *
 * @param string $pattern Pattern  chercher
 * @param string $change1 Texte devant le pattern ajout
 * @param string $change2 Texte derrire le pattern ajout
 * @param string $string Texte
 * @return string Nouveau Texte
 */
function replace($pattern, $change1, $change2, $string) {
	$last_position = 0;
	$position = 0;
	$tempstring = '';
	while(is_int($position = strpos(strtoupper($string), strtoupper($pattern), $position))) {
		$tempstring .= substr($string, $last_position, $position - $last_position);
		$tempstring .= $change1;
		$tempstring .= substr($string, $position, strlen($pattern));
		$tempstring .= $change2;
		$last_position = $position + strlen($pattern);
		$position++;
	}
	if($last_position === 0) {	// Nothing Found
		$tempstring = $string;
	} else {			// We add the end
		$tempstring .= substr($string, $last_position, strlen($string) - $last_position);
	}
	return $tempstring;
}

/**
 * Redimensionne une image si elle est trop grande
 * par rapport  sa largeur et sa hauteur.
 *
 * @param int $width Largeur Courante
 * @param int $height Hauteur Courante
 * @param int $max_width Largeur Maximale
 * @param int $max_height Hauteur Maximale
 */
function resize_image(&$width,&$height,$max_width,$max_height) {
	$ratio1 = 1;
	$ratio2 = 1;
	if($width > $max_width) {
		$ratio1 = $max_width / $width;
	}
	if($height > $max_height) {
		$ratio2 = $max_height / $height;
	}
	$ratio = min($ratio1, $ratio2);
	$width = $width * $ratio;
	$height = $height * $ratio;
}

/**
 * Ajoute des BackSlash avec la fonction addslashes  toutes
 * les valeurs de $table.
 * Cette fonction supporte le multi-table.
 *
 * @param string[] $table Tableau  ajouter les slashes
 */
function addSlash(&$table) {
	if(is_array($table)) {
		reset($table);
		while(list($key, $value) = each($table)) {
			if(is_array($table[$key])) {
				addSlash($table[$key]);
			} else {
				$table[$key] = addslashes($value);
			}
		}
		reset($table);
	}
}

/**
 * Checks if a value exists in an array by looking specifically the $member into
 * each class.
 *
 * @param mixed $needle
 * @param array $haystack
 * @param string $member
 * @param bool $strict
 */
function in_array_class($needle, $haystack, $member, $strict) {
	$c = count($haystack);

	for($i = 0; $i < $c; $i++) {
		if($strict === true) {
			if($haystack[$i]->$member === $needle) {
				return true;
			}
		} else {
			if($haystack[$i]->$member == $needle) {
				return true;
			}
		}
	}

	return false;
}

/**
 * Displays errors in RED to be sure that we will see them !
 */
function errorHandler($errno, $errstr, $errfile, $errline) {
	if(!defined(E_RECOVERABLE_ERROR)) {
		define(E_RECOVERABLE_ERROR, 4096);
	}

	$errortype = array (
		E_ERROR			=> "Error",
		E_WARNING		=> "Warning",
		E_PARSE			=> "Parsing Error",
		E_NOTICE		=> "Notice",
		E_CORE_ERROR		=> "Core Error",
		E_CORE_WARNING		=> "Core Warning",
		E_COMPILE_ERROR		=> "Compile Error",
		E_COMPILE_WARNING	=> "Compile Warning",
		E_USER_ERROR		=> "User Error",
		E_USER_WARNING		=> "User Warning",
		E_USER_NOTICE		=> "User Notice",
		E_STRICT		=> "Runtime Notice",
		E_RECOVERABLE_ERROR	=> "Recoverable Error"
		);
	echo '<div style="color: red;"><span style="font-weight: bold;">' . $errortype[$errno] . ':</span> ' . $errstr . ' in <span style="font-weight: bold;">' . $errfile . '</span> on line <span style="font-weight: bold;">' . $errline . '</div><br />';
}

/**
 * Color the line of the table where there is an error.
 * $table is the table where is every lines.
 * $table_cols contains the ordered-key of the column.
 * $error contains the error keys
 *
 * @param object $table
 * @param array $table_cols
 * @param array $error
 */
function colorError($table, $table_cols, $error) {
	if(!is_array($error))
		$error = array($error);
	$c = count($error);
	for($i = 0; $i < $c; $i++) {
		$key = array_search($error[$i], $table_cols);
		if($key !== false) {
			$table->addRowAttribute($key, 'class', 'error');
		}
	}

}
?>